#!/usr/bin/python

# Yould - A name generator generator
# This program was initially written by © Yannick Gingras - 2007

# Is was adapted by © Arne Babenhauserheide in 2007. 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

from urllib import urlopen
from re import compile

# Different compilations from gutenberg.org

# Esperanto

books_esperanto = [17665, 18178, 20802, 17482, 11511, 8177, 20931, 21194, 19183, 20178, 20162, 19858, 19803, 19030, 19182, 19293, 21195, 21951, 17425, 18326, 20943, 17945, 18836, 11307, 20006, 8224]

# marion zimmer bradley, flatland. 
books_english_science_fiction = [20796, 19726, 97]

books_english_romantic_poetry = [18500, 4800, 8209, 20158, 9622]

books_english_erotic_classic = [5225, 3726]

books_english_erotic_modern = [20028, 16885, 7889, 6852, 4300]

# King James Bible
books_english_bible = [8010, 8011, 8012, 8013, 8014, 8015, 8016, 8017, 8018, 8019,
         8001, 8020, 8021, 8022, 8023, 8024, 8025, 8026, 8027, 8028,
         8029, 8002, 8030, 8031, 8032, 8033, 8034, 8035, 8036, 8037,
         8038, 8039, 8003, 8040, 8041, 8042, 8043, 8044, 8045, 8046,
         8047, 8048, 8049, 8004, 8050, 8051, 8052, 8053, 8054, 8055,
         8056, 8057, 8058, 8059, 8005, 8060, 8061, 8062, 8063, 8064,
         8065, 8066, 8006, 8007, 8008, 8009]

BOOKS = books_esperanto
PATH = "Esperanto" #: Path inside the download-directory. 

BASE_URL = "http://www.gutenberg.org/"
MAIN_URL = BASE_URL + "etext/%d"
DL_PAT = compile(r'<td class="pgdbfilesdownload"><a href="(/(?:dirs|files)/.*?.txt)"')
MAX = 2**22

for book in BOOKS:
    try:
        data = urlopen(MAIN_URL % book).read(MAX)
        print "Book:", book
        dl = DL_PAT.findall(data)[0]
        print 'File to download:', dl
        path = "training_materials/gutenberg_downloads/" + PATH + "/%04d.txt" % book
        print "Local Path:", path
        stream = urlopen(BASE_URL + dl)
        open(path, "w").write(stream.read(MAX))
    except:
        print "Error"
    



#!/bin/env python
# This Python file uses the following encoding=utf-8

# Amo - Allgemeine Modulare Objektverwaltung im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

# Dieses Skript übersetzt tag strings der Form

# tag:url,yyyy:name

# zu Objekten der Form (YAML-notiert): 

# tag: 
#       Jahr: yyyy
#       URL: url
#       Name: Name

"""Read a tag and return a dict with the
tags information."""

class tag_zu_objekt: 
        """Read a tag and return a dict with the
tags information."""
        def __init__(self, tag): 
                self.tag = tag
        def tag_objekt(self):
                
                # Erstmal definieren wir Grundformate. 
                FORMAT_JAHR = 'yyyy'
                FORMAT_TAG = 'tag:'
                # Und deren Längen. 
                length_yyyy = len(FORMAT_JAHR)
                length_tag = len(FORMAT_TAG)
                
                # Danach schmeißen wir den Prefix raus:'t ag:'
                zwischenstring = self.tag[length_tag:]
                # Dann müssen wir in der Lage sein den String zu teilen. 
                # Dafür suchen wir erstmal das Komma im String
                ort_des_kommas = zwischenstring.find(',')
                # Um Fehler auszuschließen, prüfen wir noch, ob mehr als ein Komma im tag vorkommt. TODO: Sauber schreiben! 
                anzahl_der_kommata = zwischenstring.count( ',' )
                if anzahl_der_kommata > 1: 
                        print 'Meldung: Mehr als ein Komma im Tag! Prüfen!'
                else: #print 'Meldung: Nur ein Komma im Tag. Alles OK'
                    pass 
                # Jetzt definieren wir uns die Funktion, 
                # die aus dem String ein Objekt macht. 
                URL = zwischenstring[:ort_des_kommas]
                Jahr = zwischenstring[ort_des_kommas + 1:ort_des_kommas + length_yyyy + 1] 
                Name = zwischenstring[ort_des_kommas + length_yyyy + 2:]
                tag_dict = {}
                tag_dict['URL'] = URL
                tag_dict['Jahr'] = Jahr
                tag_dict['Name'] = Name
                return tag_dict

# und rufen sie auf. 



#!/usr/bin/env python
# encoding: utf-8

############################################################################
#    Copyright (C) 2007 by Achim Zien und Arne Babenhauserheide            #
#    arne_bab@web.de                                                       #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

# Wir müssen tags parsen können. 
import tag_zu_objekt

# Außerdem müssen wir Pfade Plattformunabhängig verbinden können. 
from os.path import join

# Wir brauchen noch den Nutzerordner. 
from user import home

BASISORDNER = home

TRENNER_ORDNER = '/'
TRENNER_VERSION = '-'
TRENNER_JAHR = '-'
TRENNER_URL = '-'
ENDUNG_DATEI = '.yml'

class Datei(): 
    def __init__(self, tag="tag:1w6.org,2007:Menschen", kategorie="Vorlagen", version="0.1", speicherordner_rel_zu_home=".amov"):
        self.tag = tag
        self.kategorie = kategorie
        self.version = version
        self.basisordner = join(BASISORDNER, speicherordner_rel_zu_home)
        self.tag_zum_objekt = tag_zu_objekt.tag_zu_objekt(self.tag)
        self.tag_objekt = self.tag_zum_objekt.tag_objekt()

    def dateipfad(self): 
        return join(self.basisordner, self.kategorie)
    
    def dateiname_ohne_version_und_pfad(self): 
        dateiname_ohne_version_und_pfad = self.tag_objekt['Name'] + TRENNER_URL + self.tag_objekt['URL'] + TRENNER_JAHR + self.tag_objekt['Jahr']
        return dateiname_ohne_version_und_pfad

    def dateiname_ohne_version(self): 
        dateiname_ohne_version = join(self.dateipfad(), self.dateiname_ohne_version_und_pfad())
        return dateiname_ohne_version

    def dateiname_ohne_version_mit_trenner(self): 
        return self.dateiname_ohne_version() + TRENNER_VERSION

    def dateiname_ausgeben(self): 
        # Mit dem Tag_objekt und dem Eingabe-Wörterbuch können wir nun den Datei-String erzeugen. 
        datei = self.dateiname_ohne_version_mit_trenner() + str(self.version) + ENDUNG_DATEI
        return datei

#### Self-Test ####

if __name__ == "__main__": 
    datei = Datei(tag="tag:1w6.org,2007:Menschen", kategorie="Vorlagen", version="0.1")
    print datei.dateiname_ausgeben()
    print datei.dateiname_ohne_version_mit_trenner()
    print datei.dateiname_ohne_version()



#!/bin/env python
# encoding: utf-8

# Charakterverwaltung - Verwalte Charaktere im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

### Imports ###

# Wir brauchen die Klasse Charakter
import Charakter

# Für die Übersetzung von IDs in Dateinamen brauchen wir noch tag_zu_datei
from Skripte.Kernskripte.ein_und_ausgabe import tag_zu_datei

# Um Charaktere per Kommandozeile übergeben zu können brauchen wir sys
import sys

# Außerdem Wollen wir einfach die neuste Version anfragen können. 
import Versionsverwaltung


### Imports ###


### Klassen ###

class Verwaltung: 
    """Manage Data-Objects. Input are the tag and the Kategorie."""
    def __init__(self, tag=u"tag:draketo.de,2007:Sskreszta", kategorie=u"Charaktere"): 
        self.tag = tag
        self.kategorie = kategorie
        #: the ID of the most recent Version. 
        self.neuste_version = self.neuste_version() 
    def neuste_version(self): 
        """return the ID of the most recent Version."""
        versionsverwaltung = Versionsverwaltung
        versionen = versionsverwaltung.Versionen(tag=tag, kategorie=u"Charaktere")
        ID = versionen.neuste #: The ID to retrieve the file.
        return ID

### Klassen ###


### User Input ###

# Das Skript muss wie folgt aufgerufen werden: 
# python charakterverwaltung.py <Kategorie> <Name> <Version> <URL> <Jahr>

try: 
    Kategorie = sys.argv[1]
    Name = sys.argv[2]
    Jahr = sys.argv[5]
    URL = sys.argv[4]
    Version = sys.argv[3]
    
    yaml_id = "ID: tag:" + URL + "," + Jahr + ":" + Name + "\n" + "Version: " + Version + "\n" + "Kategorie: " + Kategorie
    print yaml_id
    tag = "tag:" + URL + "," + Jahr + ":" + Name
    print tag
except: 
    print "Keine Kommandozeilenargumente. Verwende Standard: Sskreszta"
    yaml_id = """ID: tag:draketo.de,2007:Sskreszta
Version: 0.15
Kategorie: Charaktere
"""
    tag = "tag:draketo.de,2007:Sskreszta"

### User Input ###


### Testen ### 


verwaltung = Verwaltung(tag, u"Charaktere")

ID = verwaltung.neuste_version


charakter = Charakter.Charakter(ID)

print u'Wir haben den/die', charakter.kategorie, u'"' + charakter.name + u'" gelesen.'

char_kampfwerte = charakter.kampfwerte

print "Für das Schlachtfeld: ", char_kampfwerte

try: 
    for wert in char_kampfwerte: 
        print wert + ":", char_kampfwerte[wert]
    
    char_hauptwaffe = char_kampfwerte[u"Hauptwaffe"]
    print "\nHauptwaffe:"
    for wert in char_hauptwaffe[u"Waffe"]: 
        print wert + ":", char_hauptwaffe[u"Waffe"][wert]
    
    char_kampffertigkeit = char_hauptwaffe[u"Kampffertigkeit"]
    
    print u"Kampffertigkeit:", char_kampffertigkeit[u"Zahlenwert"]
except: 
    print "Keine Kampfwerte"

### Testen ###

## Jetzt definieren wie den Namen des Charakters
## wie er im Programm verwendet wird. 
## TODO: Beliebig viele Charakternamen 
## akzeptieren. Aktuell geht nur einer. 


#print "Öffne den Charakter " + argv[1]
#char_name = argv[1]

## Daraus erstellen wir den Dateinamen für diesen
## Charakter. 

#char_datei = char_name + "/" + char_name + ".yml"

## Danach definieren wir ein Objekt zum 
## Auslesen der Datei. 
## Achtung! Diese Objekte müssen neu instanziiert
## werden, wenn ich die Datei erneut lesen will. 

#char_auslesen = file(char_datei, 'r')

## Nun lesen wir den Charakter in einen String
## und schließen die Datei. 

#charakter_datei = char_auslesen.read()
#char_auslesen.close()


## Und übersetzen ihn mit YAML 
## in ein Python Objekt
## Das wird unser Grundlegendes Objekt werden. 

#charakter = yaml.load(charakter_datei)

## Jetzt öffnen wir eine temporäre Datei zum
## schreiben. 

#char_ausgeben = file(char_datei + '.tmp', 'w')

## schreiben den Charakter

#char_ausgeben.write(yaml.dump(charakter, default_flow_style=False, allow_unicode=True, indent=2))

## und schließen sie gleich wieder. 

#char_ausgeben.close()

## Da der Test hoffentlich geklappt hat, 
## speichern wir zum Abschluss des Programmes in 
## der Originaldatei. 

#char_speichern = file(char_datei, 'w')

#char_speichern.write(yaml.dump(charakter, default_flow_style=False, allow_unicode=True, indent=2))

#char_speichern.close()

## Abschlussbemerkung

#print "Die Bearbeitung von " + charakter[0]["Name"] + " aus der Kampagne " + charakter[2]["Grunddaten"]["Chronik"] + " ist abgeschlossen."

## print "Die gespeicherten Daten sind: "
## print yaml.dump(charakter, default_flow_style=False)



#!/bin/env python
# encoding: utf-8

# Charakterverwaltung - Verwalte Charaktere im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

"""Manage Charakter-Objects. 

Diese Datei ist ein Container für die klasse Charaktere. 

Aufrufen mit ID-Objekt oder tag-string. 

Beispiel: 
>>> Sskreszta = Charakter(tag=u"tag:draketo.de,2007:Sskreszta")
>>> Sskreszta.name
'Sskreszta'

ToDo: Größere Teile des Charaktersnicht mehr als attribut des OBjekts doppelspeichern, sondern nur durch Funktionen aufrufen. Das dict sollte nur einmal echt geladen werden. Die Eigenschaften des Charakters sollten mit Funktionen aus diesem Hauptdict herausgezogen werden. 
Ausgenommen davon sind Werrte, sie sehr oft gebraucht und geänbdert werden (Geschwindkeit statt Ram-Verbrauch). 
Aktuell wird er Charakter in etwa doppelt geladen.
 
 #TODO: Sourcefile dir hinzufügen. """

### Imports ###

# Um die Dateien zu laden, nutzen wir die Objekt-Klasse. 
import Skripte.Kernskripte.Objekt as Objekt

# Und wir brauchen die Basisklasse: Object
from Object import Object, _

# Um immer die neuste Version zu haben brauchen wir die Versionsverwaltung
from Versionsverwaltung import Versionen

### Imports ###

### Klassen ###

class Charakter(Object): 
    def __init__(self, ID=None, art=None,  tag=u"tag:draketo.de,2007:Sskreszta", kategorie="Charaktere", *args, **kwds): 
        super(Charakter, self).__init__(ID=ID, art=art,  tag=tag, kategorie=kategorie, *args, **kwds)
        self.beschreibung = self.ladeBeschreibung()
        self.werte = self.ladeWerte()
        self.eigenschaften = self.ladeEigenschaften()
        self.fertigkeiten = self.ladeFertigkeiten()
        self.kampfwerte = self.ladeKampfwerte()
        self.ausruestung = self.ladeAusruestung()
        self.schutz = self.ladeSchutz()
        self.herkunft = self.ladeHerkunft()
        self.sprache = self.ladeSprache()
        self.region = self.ladeRegion()
        self.stimmung = self.ladeStimmung()
        self.kategorie = self.objekt.kategorie
        # del self.daten # Wir müssen dieses Wörterbuch nicht löschen, 
        # da Python es zu cachen scheint, so dass wir dadurch kein Ram sparen. 
        # Vielleicht existiert es auch nur als symbolischer Link. 
            
    def ladeName(self): 
        """Lade den Namen des Charakters. 
        
        Wenn der Charakter keinen Namen hat, dann erzeuge einen zufälligen."""
        if self.daten[0][_(u"Name")] == " ": 
            self.daten[0][_(u"Name")] = self.objekt.objekt_name()
            self.objekt.write()
        return self.daten[0][_(u"Name")]
        
    def ladeGrunddaten(self): 
        """Lade die Grundlegenden Daten des Charakters, wie seine Beschreibung oder Herkunft. 
        
        Grunddaten enthalten im allgemeinen keine würfelrelevanten Werte."""
        return self.daten[1][_(u"Grunddaten")]
    
    # Daten laden
    def ladeKampfwerte(self): 
        """Lade die Kampfrelevanten Werte des Charakters."""
        return self.daten[4][_(u"Kampfwerte")]
    def ladeAusruestung(self): 
        """Lade die Ausrüstung des Charakters."""
        return self.daten[3][_(u"Ausrüstung")]
    def ladeBeschreibung(self): 
        return self.ladeGrunddaten()[_(u"Beschreibung")]
    def ladeHerkunft(self): 
        return self.ladeGrunddaten()[_(u"Herkunft")]
    def ladeRegion(self): 
        return self.herkunft[_(u"Region")]
    def ladeSprache(self): 
        return self.herkunft[_(u"Sprache")]
    def ladeStimmung(self): 
        return self.ladeGrunddaten()[_(u"Stimmung")]
    def ladeWerte(self): 
        return self.daten[2][_(u"Werte")]
    def ladeEigenschaften(self): 
        return self.werte[0][_(u"Eigenschaften")]
    def ladeFertigkeiten(self): 
        return self.werte[1][_(u"Fertigkeiten")]
    def ladeSchutz(self): 
        return self.kampfwerte[_(u"Hauptrüstung")][_("Schutz")]
    
    # Daten in das dict speichern
    # Das ist nötig, weil sonst beim umschreiben der Attribute des charakter-objekts das strings im dict nicht geändert wird (nur die Attribute werden umgeleitet - Strings sind nicht mutable. 
    def speichereKampfwerte(self, daten): 
        """Speichere die Kampfrelevanten Werte des Charakters."""
        self.daten[4][_(u"Kampfwerte")] = daten
    def speichereAusruestung(self, daten): 
        """Speichere die Ausrüstung des Charakters."""
        self.daten[3][_(u"Ausrüstung")] = daten
    def speichereBeschreibung(self, daten): 
        self.ladeGrunddaten()[_(u"Beschreibung")] = daten
    def speichereHerkunft(self, daten): 
        self.ladeGrunddaten()[_(u"Herkunft")] = daten
    def speichereRegion(self, daten): 
        self.herkunft[_(u"Region")] = daten
    def speichereSprache(self, daten): 
        self.herkunft[_(u"Sprache")] = daten
    def speichereStimmung(self, daten): 
        self.ladeGrunddaten()[_(u"Stimmung")] = daten
    def speichereWerte(self, daten): 
        self.daten[2][_(u"Werte")] = daten
    def speichereEigenschaften(self, daten): 
        self.werte[0][_(u"Eigenschaften")] = daten
    def speichereFertigkeiten(self, daten): 
        self.werte[1][_(u"Fertigkeiten")] = daten
    def speichereSchutz(self, daten): 
        self.kampfwerte[_(u"Hauptrüstung")][_("Schutz")] = daten
    def speichereName(self, daten): 
        self.daten[0][_("Name")] = daten
    
    def basic_data_yaml(self):
        return self.leerer_charakterbogen_yaml()
    
    def leerer_charakterbogen_yaml(self): 
    	return """- Name: " "
- Grunddaten: 
    Beschreibung: " "
    Herkunft: 
        Sprache: Esperanto
        Region: ' '
    Stimmung: ' '
- Werte:
    - Eigenschaften: " "
    - Fertigkeiten: 
        Nahkampf: &id001
          Zahlenwert: 12
          Grundwert: 12
    - Berufe: 
        Mensch: 
            Zahlenwert: 9
            Grundwert: 9
    - Merkmale: " "
- Ausrüstung: 
    Waffen:
      Waffenlos: &id002
        Name: Iron knuckles
        Schaden: 1
    Rüestung: 
        Stoffkleidung: &hauptruestung
            Name: Simple garb
            Schutz: 1
- Kampfwerte:  
    Hauptwaffe:
      Kampffertigkeit: *id001
      Waffe: *id002
    Trefferpunkte: 24
    Wundschwelle: 4
    Wunden: 
        körperlich tief: 0
        körperlich kritisch: 0
    Hauptrüstung: *hauptruestung
"""
    
    def save(self, name=None): 
        """Save the character into a file with its name. If the file already exists, increment the minor version number. 
        
        @param name: The name of the char (to call this function from outside - from ews).
        @type name: String """
        # raise NotImplementedException("Not yet implemented")
        # Get the name of the loaded char or template
        tagname = self.ID[_(u"ID")][self.ID[_(u"ID")].index(":", 4) + 1:]
        
        # If the name changed - self.name is the saved chars name, tagname is the name stored in the tag, name is the parameter passed to this function
        if name is not None and name != tagname or tagname != self.name: 
            # change the tag-line. 
            # if name was given, use it. comparing to None is the fastest, so we do that first. 
            if name is not None: 
                self.ID[_(u"ID")] = self.ID[_(u"ID")][:self.ID[_(u"ID")].index(":", 4) + 1] + name
            else: # name is None, self.name doesn't equal tagname
                self.ID[_(u"ID")] = self.ID[_(u"ID")][:self.ID[_(u"ID")].index(":", 4) + 1] + self.name
        
        # Now the tagline in the ID changed. 
        
        # First get the new ID
        # self.ID ist die alte ID
        # Use Versionsverwaltung to get a version object
        versionen = Versionen(tag=self.ID[_(u"ID")], kategorie=self.ID[_(u"Kategorie")])
        # Now call the version object to get the ID with a version chiw is increased by one. 
        self.ID = versionen.version_minor_eins_hoeher()
        
        # Now change the basic dict below the attributes. 
        
        self.speichereKampfwerte(self.kampfwerte)
        self.speichereAusruestung(self.ausruestung)
        self.speichereBeschreibung(self.beschreibung)
        self.speichereHerkunft(self.herkunft)
        self.speichereRegion(self.region)
        self.speichereSprache(self.sprache)
        self.speichereStimmung(self.stimmung)
        self.speichereWerte(self.werte)
        self.speichereEigenschaften(self.eigenschaften)
        self.speichereFertigkeiten(self.fertigkeiten)
        self.speichereSchutz(self.schutz)
        self.speichereName(self.name)
        
        # Now load and save the object at the same time. 
        self.objekt = Objekt.Objekt(ID=self.ID, template=self.daten)
        # TODO: Implement save chars - add name to tag, if it changed (i.e. if the char is called as template)
    

### Klassen ###


### Self-Test ###

def _test(): 
	import doctest
	doctest.testmod()

if __name__ == "__main__": 
   _test() 

if __name__ == '__main__': 
        charakter = Charakter()
        print 'Name:', charakter.name
        print 'Art:', charakter.art
        print '\nID:'
        for i in charakter.ID: 
            print '-', i + ':', charakter.ID[i]
        #print 'Objekt:', charakter.objekt
        #print 'Charakter:', charakter.charakter
        #print 'Grunddaten:', charakter.grunddaten
        #print 'Werte:', charakter.werte
        print '\nEigenschaften:'
        for i in charakter.eigenschaften: 
            print '-', i + ':', charakter.eigenschaften[i]
        print '\nFertigkeiten:'
        for i in charakter.fertigkeiten: 
            print '-', i + ':', charakter.fertigkeiten[i]
        print '\nKampfwerte:'
        for i in charakter.kampfwerte: 
            print '-', i + ':', charakter.kampfwerte[i]
        print '- Schutz:', charakter.schutz
        print '\nBeschreibung:', charakter.beschreibung
        print 'Herkunft:', charakter.herkunft
        print 'Sprache:', charakter.sprache
        print 'Region:', charakter.region
        print 'Stimmung:', charakter.stimmung
        print 'Kategorie:', charakter.kategorie
        
        # Test saving
        # First change the Beschreibung
        charakter.beschreibung += ". Zusatz 1."
        print charakter.charakter
        # Save the character with an incremented minor version
        charakter.save()

### Self-Test ###



#!/bin/env python
# encoding: utf-8 

# Schlachtfeld - Großkämpfe im EWS System 
#   http://rpg-tools-1d6.sf.net
# Copyright © 2007 - 2007 Achim Zien

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA


'''Return localizable strings. 
'''


#### IMPORTS ####

# Obtain Charakters from yaml-files
from amov import Lokalisierung, Versionsverwaltung
# replaced the alias with the real directory. 

#### IMPORTS ####


#### CLASSES ####

class Locale(Lokalisierung.Lokalisierung):
    """Localized Strings.
    
    *args and **kwds makes sure, that we aren't confused, when we get more arguments than we expected."""
    def __init__(self,source="tag:1w6.org,2008:en_EN", *args,  **kwds):
        '''Basic creation of a single Char.'''
        # First call super, so that miltiple inheritance with char as first class can work. 
        super(Locale, self).__init__(tag=source, *args,  **kwds)
    
    def basic_data_yaml(self):
        """Return basic localizable Strings"""
        return """Name: en_EN

Input: 
  Please identify yourself: "Please type an email address or domain name as identifier for you as player: "
  Please give a name: "Please type a name for your char: "

Story:
  Greeter_1: Greetings,
  Greeter_2: You will have to prevail in this battle.
  Greeter_3: The first strike shall be yours.
  Greeter_4: Please select with the arrow keys or the number pad where you want to
    attack. Confirm with Enter or Return.
  Cease fighting: You had to cease fighting.
  You: You 
  comma: ", "
  Scored a hit: ", scored a hit."
  Your enemy: "Your enemy "
  took: " took "
  a deep wound: " a deep wound." 
  a critical wound: " a critical wound."
  You should stop: "You should stop this battle!"
  You take: ", take "
  Points of damage: " points of damage."
  Your turn to defend: Now it's your turn to defend yourself. Decide where you want to defend and counterattack.
  Your turn to attack: Now it's your turn to attack. Decide where you want to attack.

Battle Details: 
  Greeter_1: Details about the Battle
  First position set 1: First position set as 
  First position set 2: " - Counter position still needs to be set."
  Pos mod info 1: "Own position change mod: "
  Pos mod info sep 1: " [ "
  Pos mod info sep 2: " , "
  Pos mod info sep 3: " ] "
  Pos mod info 2: ", the others pos change mod: "
  Own position info 1: "self position_1: "
  Other position info 1: ", other position_1: "
  Attack prediction info: ", attack prediction modifier for defender: "
  Name-Role sep: ", "
  Attack-Choice sep: "-"
  Attack result info: " - attack result: "
  Weapon damage info: ", damage: "
  Armor value info: ", armor: "
  Fights against: fights against
  
Banner: 
  Game Over: Game Over
  Lose: You lost. 
  Win: You won. 
    
"""


#### CLASSES ####

#### Self-Test ####

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__": 
    _test()
    locale = Locale()
    print locale.locale
    
#### Self-Test ####



#!/bin/env python
# encoding: utf-8

# Armeenverwaltung - Verwalte Armeen im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

"""Verwalte Dateien von Armeen

Beispiel: 
>>> Schlacht = Schlacht()
>>> Schlacht.schlacht
[(('Menschen Armee', 0.5, 0.40000000000000002), [], [('tag:1w6.org,2007:Mensch', 10, -1, 'Soldat', True), ('tag:draketo.de,2007:Sskreszta', 1, -1, 'Held', False)]), (('Goblin Armee', 0.40000000000000002, 0.40000000000000002), [], [('tag:1w6.org,2007:Goblin', 15, -1, 'Soldat', True)])]

 """


### Imports ###

# Für das Dateiformat brauchen wir yaml
from yaml import load as yaml_load

# Um die Dateien zu laden, nutzen wir die Objekt-Klasse. 
import Skripte.Kernskripte.Objekt as Objekt

# Um immer die neuste Version zu haben brauchen wir die Versionsverwaltung
from Versionsverwaltung import Versionen

### Imports ###

### Klassen ###

class Schlacht: 
    def __init__(self, ID=None, art=None,  tag=u"tag:1w6.org,2008:Menschen_gegen_Goblins",  data=None): 
        # TODO: erstelle eine Grundklasse, die die laden-methode liefert. Diese Grundklasse sollte auch den parameter ketegorie haben, so dass sie auch __init__ komplett übernehmen kann. 
        #: Der Identifikator der Armee-Datei
        if ID is not None: 
            self.ID = ID
        elif tag is not None: 
            # Wenn keine ID bekannt ist, lade die neuste Datei
            versionen = Versionen(tag=tag,  kategorie=u"Schlachten")
            self.ID = versionen.neuste
        #: Die Art. Wird noch nicht verwendet. 
        self.art = art
        if data is not None: 
            self.data = data
            # TODO: If data is given, create a new version of the file. 
        
        #: Das Wörterbuch, das die Daten enthält. Kann (da vollständiges yaml) auch echte objekte enthalten. 
        self.objekt = self.laden() 
        
        
    def laden(self):
        """Lade die benötigten Daten aus den gegebenen Parametern."""
        
        #: Das Objekt. Es bietet das dict und ein paar Methoden. 
        self.objekt_tmp = Objekt.Objekt(ID=self.ID, template=yaml_load(self.standard_yaml()))
        #: Das Wörterbuch mit allen Charakterdaten aus der Datei. 
        return self.objekt_tmp.objekt
    
    def standard_yaml(self): 
    	return """# Battlefield definition: (army, [groups], [chartypes])
    # Army definition: ('name', resignratio, fleeratio)
    # Chartype definition: ("tag", number_of_soldiers, 'group' (-1=army), 'Type_of_chars', from_template?)
- !!python/tuple
  - !!python/tuple
    - Menschen Armee
    - 0.5
    - 0.40000000000000002
  - []
  - - !!python/tuple
      - tag:1w6.org,2007:Mensch
      - 10
      - -1
      - Soldat
      - true
    - !!python/tuple
      - tag:draketo.de,2007:Sskreszta
      - 1
      - -1
      - Held
      - false
- !!python/tuple
  - !!python/tuple
    - Goblin Armee
    - 0.40000000000000002
    - 0.40000000000000002
  - []
  - - !!python/tuple
      - tag:1w6.org,2007:Goblin
      - 15
      - -1
      - Soldat
      - true

"""
    
    

### Klassen ###

### Self-Test ###

def _test(): 
	import doctest
	doctest.testmod()

if __name__ == "__main__": 
   _test() 

if __name__ == '__main__': 
        pass

### Self-Test ###
        



#!/bin/env python
# encoding: utf-8

# Charakterverwaltung - Verwalte Charaktere im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

"""Manage Grammars for Name-Generation. 

This file returns a grammar-object if it is given an ID or tag. 
Useage: 

>>> grammar = Grammar(ID=None, art=type, tag="tag:1w6.org,2007:Foo")
>>> print "The Grammar is:", grammar.grammar
The Grammar is: {'nameMiddle': ['<nmCons><nmVowel>'], 'nameStart': ['<nsCons><nmVowel>', '<nsCons><nmVowel>', '<nsCons><nmVowel>', '<nsVowel>'], 'nameMiddle0to2': ['', '<nameMiddle>', '<nameMiddle><nameMiddle>'], 'name': ['<nameStart><nameMiddle0to2><nameEnd>'], 'neVowel': ['e', 'i', 'a', 'au'], 'nmCons': ['l', 'm', 'lm', 'th', 'r', 's', 'ss', 'p', 'f', 'mb', 'b', 'lb', 'd', 'lf'], 'nsVowel': ['A', 'Au', 'Ei'], 'nsCons': ['J', 'M', 'P', 'N', 'Y', 'D', 'F'], 'nmVowel': ['a', 'e', 'i', 'o', 'u', 'au', 'oa', 'ei'], 'neCons': ['r', 'n', 'm', 's', 'y', 'l', 'th', 'b', 'lb', 'f', 'lf'], 'nameEnd': ['<neCons><neVowel>', '<neCons>', '<neCons>']}



You need either an ID or a tag. If you use a tag, you must set the ID to None, else teh default ID will be used: 

grammar = Grammar(ID=None, art=type, tag=tag)"""

### Imports ###

# Für das Dateiformat brauchen wir yaml, hier müssen wir yaml nur laden können. 
from yaml import load as yaml_load

# Um die Dateien zu laden, nutzen wir die Objekt-Klasse. 
import Skripte.Kernskripte.Objekt as Objekt

# Außerdem brauchen wir die Versionsverwaltung.
import Versionsverwaltung

### Imports ###

### Klassen ###

class Grammar: 
    def __init__(self, ID=None, art=None, tag="tag:1w6.org,2007:Foo"): 
        # If we only get a tag and no ID, use Versionsverwaltung, to create an ID from the tag.
        if ID != None: 
            self.ID = ID
        else: 
            versionen = Versionsverwaltung.Versionen(tag=tag,kategorie=u"Namensgrammatiken")
            self.ID = versionen.neuste
        #: The kind of name we want to generate. This isn't used yet.
        self.art = art
        # Utilize the Objekt coremodule to load the Grammar Object from a file via the ID. Also supply a template for the case that the file doesn't exist. 
        self.objekt = Objekt.Objekt(ID=self.ID, template=yaml_load(self.grammar_template_yaml()))
        # Get self.grammer, the content of the object which is the attribute we use for name-generation.
        self.grammar = self.objekt.objekt
    
    def grammar_template_yaml(self): 
        """Return a grammar template."""
        return """name: [<nameStart><nameMiddle0to2><nameEnd>]
nameEnd: [<neCons><neVowel>, <neCons>, <neCons>]
nameMiddle: [<nmCons><nmVowel>]
nameMiddle0to2: ['', <nameMiddle>, <nameMiddle><nameMiddle>]
nameStart: [<nsCons><nmVowel>, <nsCons><nmVowel>, <nsCons><nmVowel>, <nsVowel>]
neCons: [r, n, m, s, y, l, th, b, lb, f, lf]
neVowel: [e, i, a, au]
nmCons: [l, m, lm, th, r, s, ss, p, f, mb, b, lb, d, lf]
nmVowel: [a, e, i, o, u, au, oa, ei]
nsCons: [J, M, P, N, Y, D, F]
nsVowel: [A, Au, Ei]
"""

#### Classes ####

#### Self-Test ####

if __name__ == '__main__': 
    # Erst holen wir uns das Namensgrammatik-Modul mit Standardwerten.
    #: Das Grammatikobjekt 
    grammar = Grammar()
    # Dann geben wir dei Grammatik aus. 
    print grammar.grammar
    # Danach testen wir das Modul mittels Docstrings. (Infos im pydoc String am Anfang der Datei)
    # Dafür holen wir uns das doctest modul. 
    import doctest
    # und starten dann dessen Test. 
    doctest.testmod()

#### Self-Test ####



#!/usr/bin/python

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

import sys
from random import seed, random
from time import time
from os import popen
from sets import Set
seed(time())

def min_nb_doms_p(name, nb):
    "true if at least nb domains are available"
    if nb == 0:
        return True
    nb_doms = 0
    for dom in (".info", ".org", ".net", ".com"):
        if len(popen("host -C "+name+dom+" 2>&1").read()) == 0:
            nb_doms += 1
            if nb_doms >= nb:
                return True
    return False

def dom_avails(name):
    # not completly good, we only check the SOA but that should do the
    # trick
    doms = []
    for dom in (".com", ".net", ".org", ".info"):
        if len(popen("host -C "+name+dom+" 2>&1").read()) == 0:
            doms.append(dom)
    return doms

def get_c(probs, draw):
    tot = 0
    for prob, c in probs:
        tot += prob
        if tot >= draw:
            return c
    return c

def gen3_name(tr3_prob):
    choices = []
    pprev = prev = None
    c = get_c(tr3_prob[(None, None)], random())
    while c != None:
        choices.append(c)
        pprev = prev
        prev = c
        c = get_c(tr3_prob[(pprev, prev)], random())
        
    return "".join(choices)

def name3_prob(tr3_prob, name):
    prob = 1.0
    pprev = prev = None
    for c in name:
        prob *= filter(lambda rec:rec[1] == c,
                       tr3_prob[(pprev, prev)])[0][0]
        pprev = prev
        prev = c
    return prob * filter(lambda rec:rec[1] == None,
                         tr3_prob[(pprev, prev)])[0][0]

if __name__ == "__main__":
    if len(sys.argv) != 6:
        print "USAGE: %s PROBS MIN_L MAX_L NB_W NB_D" % sys.argv[0]
        print "  PROBS is the language transition table"
        print "  MIN_L is the minimum desired word length"
        print "  MAX_L is the maximum desired word length"
        print "  NB_W  is the number of words to generate"
        print "  NB_D  is the number of free domain required (0 to 4)"
        sys.exit(1)
    tr3_prob = eval(open(sys.argv[1]).read())
    min_l = int(sys.argv[2])
    max_l = int(sys.argv[3])
    nb_w  = int(sys.argv[4])
    nb_d  = int(sys.argv[5])
    
    names = []
    tried = Set()
    while len(names) < nb_w:
        name = gen3_name(tr3_prob)
        if min_l <= len(name) <= max_l \
               and name not in tried \
               and name3_prob(tr3_prob, name) > 0.000000025 \
               and min_nb_doms_p(name, nb_d):
            print "%.010f: %s" % (name3_prob(tr3_prob, name), name)
            names.append(name)
        tried.add(name)

    # TODO: fix the scoring 
    #for prob, name in sorted([(name3_prob(tr3_prob, name), name)
    #                          for name in names]):
    #    print "%.010f: %s" % (name3_prob(tr3_prob, name), name)
    print "tried", len(tried), "names"
    print ", ".join(sorted(names))
    



#!/bin/env python
# encoding: utf-8

# Charakterverwaltung - Verwalte Charaktere im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

"""Manage localized strings

Hier werden lokalisierte Ausgaben geladen. 

"""

### Imports ###

# Für das Dateiformat brauchen wir yaml, in dieser Datei müssen wir sie nur laden können. 
from yaml import load as yaml_load

# Um die Dateien zu laden, nutzen wir die Objekt-Klasse. 
import Skripte.Kernskripte.Objekt as Objekt

# Um immer die neuste Version zu haben brauchen wir die Versionsverwaltung
from Versionsverwaltung import Versionen

### Imports ###

### Klassen ###

class Object(object): 
    def __init__(self, ID=None, art=None,  tag=u"tag:1w6.org,2008:de_DE", kategorie="locales", *arg, **kwds): 
        #: Der Identifikator der Charakter-Datei
        if ID is not None: 
            self.ID = ID
        elif tag is not None: 
            # Wenn keine ID bekannt ist, lade die neuste Datei
            versionen = Versionen(tag=tag,  kategorie=kategorie)
            self.ID = versionen.neuste
        # Get the name of the loaded char or template
        self.tagname = self.ID[_(u"ID")][self.ID[_(u"ID")].index(":", 4) + 1:]
        self.tagstring = self.ID[_(u"ID")]
        self.tagstring_without_name = self.ID[_(u"ID")][:self.ID[_(u"ID")].index(":", 4) + 1]
        #: Die Art der locale. Wird noch nicht verwendet. 
        self.art = art
        #: Das Charakterobjekt. Es bietet das dict und ein paar Methoden. 
        self.objekt = Objekt.Objekt(ID=self.ID, template=yaml_load(self.basic_data_yaml()))
        #: Das Charakterwörterbuch mit allen Charakterdaten aus der Datei. 
        self.daten = self.objekt.objekt
        #: Der Name der Datei
        self.name = self.ladeName()
    
    def basic_data_yaml(self): 
    	raise Exception("Must be implemented in subclass!")
    
    def ladeName(self): 
        """Lade den Namen des Charakters. 
        
        Wenn der Charakter keinen Namen hat, dann erzeuge einen zufälligen."""
        try: name = self.self.daten[_(u"Name")]
        except: 
            self.daten[_(u"Name")] = self.tagname
            name = self.tagname
        return name
    
    def save(self, name=None): 
        """Save the locale into a file with its name. If the file already exists, increment the minor version number. 
        
        @param name: The name of the char (to call this function from outside - from ews). 
        @type name: String """
        
        # If the name changed - self.name is the saved chars name, tagname is the name stored in the tag, name is the parameter passed to this function
        if name is not None and name != self.tagname or self.tagname != self.name: 
            # change the tag-line. 
            # if name was given, use it. comparing to None is the fastest, so we do that first. 
            if name is not None: 
                self.ID[_(u"ID")] = self.ID[_(u"ID")][:self.ID[_(u"ID")].index(":", 4) + 1] + name
            else: # name is None, self.name doesn't equal tagname
                self.ID[_(u"ID")] = self.ID[_(u"ID")][:self.ID[_(u"ID")].index(":", 4) + 1] + self.name
        
        # Now the tagline in the ID changed. 
        
        # First get the new ID
        # self.ID ist die alte ID
        # Use Versionsverwaltung to get a version object
        versionen = Versionen(tag=self.ID[_(u"ID")], kategorie=self.ID[_(u"Kategorie")])
        # Now call the version object to get the ID with a version which is increased by one. 
        self.ID = versionen.version_minor_eins_hoeher()
        
        # Now change the basic dict below the attributes. 
        
        # Now load and save the object at the same time. 
        self.objekt = Objekt.Objekt(ID=self.ID, template=self.daten)
        
    

### Klassen ###

### Functions ###

def _(data): 
    """Encapsulation for localization."""
    return data

### Functions ###

### Self-Test ###

def _test(): 
	import doctest
	doctest.testmod()

if __name__ == "__main__": 
   _test() 

if __name__ == '__main__': 
    pass 
    
### Self-Test ###



#!/usr/bin/python
# encoding: utf-8

# Yould - A name generator generator
# This program was initially written by © Yannick Gingras - 2007

# Is was adapted by © Arne Babenhauserheide in 2007. 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

# Adapted, so that it can also take non-gutenberg texts

import sys

from string import ascii_lowercase
from pprint import pprint

ONLY_READ_GUTENBERG_TEXTS = True

ltrs = tuple(ascii_lowercase)

BODY_TAG = " OF THE PROJECT GUTENBERG EBOOK"

MIN_W = 5
MAX_W = 14

def tokenize(line):
    "return the list of 'good' words on this line"
    words = []
    start = 0
    for i in range(len(line)-1):
        if line[i+1] not in ltrs:
            if line[start] in ltrs:
                word = line[start:i+1]
                if MIN_W <= len(word) <= MAX_W:
                    words.append(word)
                start = i+1
        elif line[start] not in ltrs:
            start = i+1
    return words
            

if __name__ == "__main__":
    tr3_freq = {} # ex: trs_freq[('l', 'a')] = (12, {'a':8, 'c':4})
    prev = pprev = None

    # train
    for f in sys.argv:
        in_body = False
        for line in open(f):
            # The following makes sure that we only read the Text from Gutenberg-ebooks, but makes this script ignore any non-gutenberg texts. 
            if ONLY_READ_GUTENBERG_TEXTS: 
                if not in_body:
                    if line.find("START"+BODY_TAG) == -1:
                        continue
                    else:
                        in_body = True
                if line.find("END"+BODY_TAG) != -1:
                    break

            for word in tokenize(line.lower()+"\n"):
                prev = pprev = None
                for c in word:
                    nb_trs, trs_h  = tr3_freq.get((pprev, prev), (0, {}))
                    trs_h[c] = trs_h.get(c, 0) + 1
                    tr3_freq[(pprev, prev)] = (nb_trs+1, trs_h)

                    pprev = prev
                    prev = c
                nb_trs, trs_h  = tr3_freq.get((pprev, prev), (0, {}))
                trs_h[None] = trs_h.get(None, 0) + 1
                tr3_freq[(pprev, prev)] = (nb_trs+1, trs_h)
                            
    # build prob tables
    # ex: trs_prob[('c', 'r')] = [(.1, 'e'), (.07, 'a'),...,(.001, 'z')]
    tr3_prob = {}
    for k in tr3_freq.keys():
        nb_trs, trs_h = tr3_freq[k]
        tr3_prob[k] = [ (1.0*freq/nb_trs, nc)
                        for nc, freq in trs_h.items()]
        tr3_prob[k].sort()
        tr3_prob[k].reverse()

    pprint(tr3_prob)



#!/bin/env python
# encoding: utf-8

# Schlachtfeld - Großkämpfe im EWS System
#   http://rpg-tools-1d6.sf.net
# Copyright © 2007 - 2007 Achim Zien

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

"""Dieses Modul kapselt mehrere Namensgeneratoren. 

Anwendung: 

import namen

name = Name(anzahl=200,  quelle="yould", art="Englisch")

Möglichkeiten: 

anzahl: yould und cfnamegen erstellen beide immer 500 namen auf einen Schlag, was bei Geschiwndigkeitstest berücksichtigt werden sollte. 

quellen: 
	- yould: Namen auf Grundlage der Wahrscheinlichkeit, dass eine bestimmte Kombination in einem Wort auftaucht. 
	- pointon_phonetic: Einfacher Namensgenerator mit zufallsalgorythmus
	- cfnamegen: Ähnlich ie pointon_phonetic. Auf Grundlage von Kontexctfreien Grammatiken 

art: Wird bisher nur von yould verwendet. Esperanto oder Enlisch. Sonstige Werte werden als "Altenglisch" gewertet (default). 

"""

#### Imports ####

import random_phonetic_name_generator_von_Pointon.random_phonetic_name_generator as phonetic_gen

import yould.yould as yould

# Get os for reading the path to the sourcefile.
from os.path import join,  dirname

import namegen.pycfnamegen.cfnamegen as cfnamegen

# Obtain Grammars from yaml-files
from amov import Namensgrammatik, Versionsverwaltung


#### Imports ####

#### Classes ####

class Name: 
    def __init__(self, art="Esperanto", familie="default", stimmung="default", anzahl=1, quelle="yould"): 
        """Container for all name-generation actions. Call it global, to be able to reuse it."""
        # TODO: Implement familie und stimmung
        self.name = "" #: One result
        self.names = [] #: The list of names.
        self.art = art #: The type of the name. Ork, Elf, Mensch, Troll, Deutsch, Englisch, ... 
        self.familie = familie #: Die Unterart. Orks -> Mordor, Orks -> Isengard, noch nicht in Verwendung
        self.stimmung = stimmung #: Stimmung: Düster, Stark, Verloren... noch nicht in Verwendug.
        self.anzahl = anzahl #: Number of names to generate. 
        self.anzahl_erzeugter_namen = 0 #: Wieviele Namen dieses Objekt bereits erzeugt hat.
        self.Quelle = quelle #: Die genutzte Namensquelle.
        
    def erzeuge(self,art="Esperanto", familie="default", stimmung="default"): 
        """Erzeuge die Namen. After 2500 names, always switch to the quick gen."""
        # Wenn diese Funktion mit veränderter Namensart aufgerufen wird, muss die Namensliste geleert werden. 
        if self.art != art: 
            self.names = []
            self.art = art
        if self.anzahl_erzeugter_namen < 4500: 
            self.schnell = False
        else: 
            self.schnell = True
        
        if self.Quelle == "pointon_phonetic" or self.schnell: 
            return self.pointon_namen()
        elif self.Quelle == "yould": 
            return self.yould_namen(art, familie, stimmung)
        elif self.Quelle == "cfnamegen": 
            return self.cfnamegen_namen()
    
    def erzeuge_speziellen_namen(self,art="Esperanto", familie="default", stimmung="default"): 
        """Erzeuge besonderen Namen. Nimm dafür immer den besten Namensgenerator."""
        return self.yould_namen(art=art, familie=familie, stimmung=stimmung)
        
    def yould_namen(self, art="Esperanto", familie="default", stimmung="default"): 
        """Hol die Namen aus dem Yould Namensgegerator."""
        if self.anzahl == 1: # Generate only one name
            # since yould is much faster when it generates many names at once, we first prepare 1000 names for internal use.
            # But only, if we have no more names avaible. 
            if len(self.names) < 2: 
                # print "Erzeuge die Namen", self.anzahl_erzeugter_namen, 'bis',  self.anzahl_erzeugter_namen + 500
                self.names = self.yould_aufrufen(anzahl=100, art=art, familie=familie, stimmung=stimmung)
                self.anzahl_erzeugter_namen += 50
            self.name = " ".join([self.names[0].capitalize(), self.names[1].capitalize()])
            # Lösche die beiden verwendeten Namen.
            del(self.names[0])
            del(self.names[0])
            # print self.name, "noch", len(self.names), "Namen verfügbar"
            return self.name
        else: # Generate many names
            self.names = self.yould_aufrufen(self.anzahl * 2, art=art, familie=familie, stimmung=stimmung)
            names = []
            for i in range(self.anzahl): 
                names.append(self.names[2*i].capitalize() + " " + self.names[(2*i)+1].capitalize())
            return names
            
    def yould_aufrufen(self, anzahl=1000, art="Esperanto", familie="default", stimmung="default"): 
        if art == "Esperanto": 
            names = yould.generate_names(eval(open(join(dirname(__file__), "yould/gutenberg_esperanto2.prob")).read()), 5, 12, 1000, 0, min_prob=0.000000001, max_prob=0.00001) # TODO: Get the path to the your dir dynamically, independent of the calling script. 
        elif art == "Englisch": 
            names = yould.generate_names(eval(open(join(dirname(__file__), "yould/various_gutenberg.prob")).read()), 5, 12, 1000, 0, min_prob=0.000000001, max_prob=0.00001)
        else: 
            names = yould.generate_names(eval(open(join(dirname(__file__), "yould/gutenberg_erotik_und_romantik.prob")).read()), 5, 12, 1000, 0, min_prob=0.000000001, max_prob=0.00001)
        return names
    
    def pointon_namen(self): 
        """Hol die Namen aus dem Pointon Namensgenertor."""
        if self.anzahl == 1: 
            self.name = phonetic_gen.namegen() + " " + phonetic_gen.namegen()
            return self.name
        else: 
            while self.anzahl > 0: 
                self.name = phonetic_gen.namegen() + " " + phonetic_gen.namegen()
                self.names.append(self.name)
                self.anzahl -= 1
            return self.names
            
    def cfnamegen_namen(self): 
        """Hol die Namen aus dem CFNameGen Namensgenertor."""
        if self.anzahl == 1: 
            # Fist create 1000 names and keep them (optimization). 
            if len(self.names) == 0 or len(self.names) == 1: 
                self.names =  self.cfnamegen_aufrufen(anzahl=1000)
            # Then for each name join two names to get first and second name. 
            self.name = self.names[0] + " " + self.names[1]
            del self.names[0]
            del self.names[0]
            return self.name
        else: 
            # return a fixed number of names.
            grammatikObjekt = Namensgrammatik.Grammar()
            fooGrammar = grammatikObjekt.grammar
            names = []
            self.names = self.cfnamegen_aufrufen(anzahl=self.anzahl*2)
            while self.anzahl != 0: 
                self.name = self.names[0] + " " + self.names[1]
                del self.names[0]
                del self.names[0]
                names.append(self.name)
                self.anzahl -= 1
            return names
        
    def cfnamegen_aufrufen(self, anzahl=1000): 
        grammatikObjekt = Namensgrammatik.Grammar()
        fooGrammar = grammatikObjekt.grammar
        foong = cfnamegen.CFNameGen(fooGrammar)
        names = []
        while anzahl != 0: 
            names.append(foong.getName())
            anzahl -= 1
        return names

        

#### Classes ####

#### Self-Test ####

if __name__ == "__main__": 
    print "Erzeuge Esperanto-Namen mit yould"
    name = Name(anzahl=20,  quelle="yould")
    namen = name.erzeuge()
    for i in namen: 
        print i
    print "\nErzeuge Englische Namen mit yould"
    name = Name(anzahl=20,  quelle="yould", art="Englisch")
    namen = name.erzeuge()
    for i in namen: 
        print i
    print "\nErzeuge erotische Namen mit yould"
    name = Name(anzahl=20,  quelle="yould", art="Martin Luther")
    namen = name.erzeuge()
    for i in namen: 
        print i
    print "\nErzeuge die Namen mit pointon"
    name = Name(anzahl=20,  quelle="pointon_phonetic")
    namen = name.erzeuge()
    for i in namen: 
        print i
    print "\nErzeuge die Namen mit cfnamegen"
    name = Name(anzahl=20,  quelle="cfnamegen")
    namen = name.erzeuge()
    for i in namen: 
        print i

#### Self-Test ####



#!/bin/env python
# encoding: utf-8

# Charakterverwaltung - Verwalte Charaktere im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

"""Manage Versions of saved objects. 

This class gets a tag and a cathegory and returns the full ID (including the version). Likely it will mostly be used to retrieve the most recent Version. """

#### Imports ####

import os

# Um die Dateien zu laden, nutzen wir die Objekt-Klasse. 
import Skripte.Kernskripte.ein_und_ausgabe.tag_zu_datei_von_tag as tag_zu_datei

#### Imports ####

#### Constants ####

TRENNER_ORDNER = "/"

#### Constants ####

#### Classes ####

class Versionen: 
    """Manage Versions. 
    
    ToDo: Rewrite, so that data about the file-path is taken from an ID object again (self.finde_neuste_datei())."""
    def __init__(self, tag=u"tag:1w6.org,2007:Menschen", kategorie=u"Armeen"): 
        #: the unique identifier for content, barring the cathegory. 
        self.tag = tag 
        #: The type of objekt we want to retrieve. 
        self.kategorie = kategorie 
        #: The ID including the most recent version of the objekt. 
        self.neuste = self.neuste_version() 
        #: The filepacth to the file with the most recent version. ToDo: Check and Remove, since this breaks the levels, which state that on this level only tag strings and ID dicts should be exchanged. 
        self.neuste_datei = self.lade_neuste_datei()

    def liste_quelldateien_auf(self, name, pfad): 
        result_list = []
        # print "name =", name, ", pfad =", pfad
        # list all files. 
        if not os.path.isdir(pfad): 
            os.makedirs(pfad)
        for i in os.listdir(pfad):
            # If the file begins with the name and ends with ".yml" add it to the list of matching files. 
            if i.find(name, 0, len(name)) != -1 and i[-4:] == ".yml":
                result_list.append(pfad + TRENNER_ORDNER + i) # TODO: Use os.path.join()
        return result_list

    def finde_neuste_datei(self, tag, pfad):
        """Return the YAML file (suffix: ".yml") in path which begins with name which has the highest version number (sorted by [].sort )."""
        datei = tag_zu_datei.Datei(tag=tag, kategorie=pfad)
        name = datei.dateiname_ohne_version_und_pfad()
        result_list = self.liste_quelldateien_auf(name, pfad)
        #: Die neuste Datei. Wenn es keine Datei gibt: False
        neuste_datei = False
        # return the one with the highest version number (major.minor) from the list of matching files. 
        #: A list containing the filepath and the major and minor versions of each file. 
        versions_list = []
        # create a versions array which contains the file path, the major versiona nd the minor version. 
        for i in result_list: 
            #: The major version of the object
            hauptversion = self.hauptversion(self.version_von_pfad(i))
            #: the minor version of the object
            unterversion = self.unterversion(self.version_von_pfad(i))
            versions_list.append((i, hauptversion, unterversion))
        # find the file with the highest version by comparing all against a highest version array and recording always the highest version. 
        #: The highest recorded version (major.minor). 
        hoechste_version = [0,0]
        for i in versions_list: 
            # if the major version is higher than the last recorded major version, this is the most recent file and we set et as neuste_datei. 
            if i[1] > hoechste_version[0]: 
                hoechste_version[0] = i[1]
                #: The most recent file. 
                neuste_datei = i[0]
            # if the major version is equal to the highest recorded major and the minor versino is higher than the last recorded minor version, set this as the most recent file. 
            elif i[1] == hoechste_version[0] and i[2] > hoechste_version[1]: 
                hoechste_version[1] = i[2]
                neuste_datei = i[0]
        if neuste_datei: 
            return neuste_datei
        else: 
            return False
    
    def lade_neuste_datei(self): 
        """Return the most recent file (by versionnumber) which matches tag and ID."""
        #: The most recent file
        neuste_datei = self.finde_neuste_datei(self.tag, self.kategorie)
        return neuste_datei
    
    def neuste_version(self):
        """return the most recent version in an ID dict."""
        #: The ID dict
        ID = {}
        ID[u"ID"] = self.tag
        ID[u"Kategorie"] = self.kategorie
        neuste_datei = self.lade_neuste_datei()
        # Set the range we found in the version-part of the filename as version. If no file exists yet, use Version 0.0
        if neuste_datei: 
            ID[u"Version"] = self.version_von_pfad(neuste_datei)
        else: 
            ID[u"Version"] = 0.0
        return ID
        
    def version_minor_eins_hoeher(self): 
        """return an ID with a minor version 1 higher than any already existing file."""
        # First get the most recent ID
        ID = self.neuste_version()
        # Then get the version
        version = ID[u"Version"]
        # Now get the major and minor version
        hauptvers = self.hauptversion(version)
        untervers = self.unterversion(version)
        # Increment the minor versino by 1
        untervers += 1
        # construct the version again
        version = str(hauptvers) + "." + str(untervers)
        # And change the ID
        ID[u"Version"] = version
        # And return it. 
        return ID
        
    
    def version_von_pfad(self, pfad): 
        """Return the version of the file whoose path was referenced."""
        # Check every char backwards from the suffix, if it is a number or a dot. 
        i = -4
        while self.is_number(pfad[i]) or pfad[i] == ".": 
            i-= 1
        return pfad[i+1:-4]

    def hauptversion(self, version): 
        """Return the major version number."""
        vers = str(version)
        i = 0
        while self.is_number(vers[i]): 
            i += 1
        return int(vers[:i])

    def unterversion(self, version): 
        """Return the major version number."""
        vers = str(version)
        i = -1
        while self.is_number(vers[i]): 
            i -= 1
        return int(vers[i+1:])
    
    def is_number(self, i): 
        """Test if the provided char/string/int is a number."""
        for j in str(range(10)): 
            if i == j: 
                return True
        return False
        
        

#### Classes ####

#### Self-Test ####

if __name__ == "__main__": 
    from yaml import dump
    versionen = Versionen()
    print "Datei:", versionen.neuste_datei
    print "ID:\n", dump(versionen.neuste, default_flow_style=False)

#### Self-Test ####



#!/bin/env python
# This Python file uses the following encoding=utf-8

# Schlachtfeld - Großkämpfe im EWS System
#   http://rpg-tools-1d6.sf.net
# Copyright © 2007 - 2007 Achim Zien

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

"""Do tests/checks (skills, etc.) according to the ±W6 rules => Throw a die."""


#### IMPORTS ####
from random import randint as rnd
#### IMPORTS ####


#### FUNCTIONS ####

def pmw6():
    """Throw the ±W6/±D6 die."""
    tmp = rnd(0,5)
    count = 1
    if tmp in [0,5]:
            while tmp == rnd(0,5):
                    count += 1
    wurf = count*ews[tmp]
    return wurf

def check(skill, MW):
    """Check if the throw reaches a target number (MW)."""
    if skill + pmw6() >= MW:
        return True
    else:
        return False
                
def open_check(skill, MW):
    """Check by how far we manage to beat the target number, or by how far we lose to it."""
    return skill - MW + pmw6()
    
def open_check_with_result(skill,  MW):
    """Check, by how far we beat (or get beaten by) a target number and also return, which absolute value we reach."""
    res = skill + pmw6()
    return res - MW,  res

#### FUNCTIONS ####


#### GAME PARS ####
ews = [-5, -3, -1, 2, 4, 6]
#### GAME PARS ####


#### SELF CHECK ####
if __name__ == '__main__':
        store = []
        for i in range(100000):
                store.append(pmw6())
        print 'min max'
        print min(store), max(store)
#### SELF CHECK ####



#!/bin/env python
# encoding: utf-8

# Charakterverwaltung - Verwalte Charaktere im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

"""Manage localized strings

Hier werden lokalisierte Ausgaben geladen. 

"""

### Imports ###

# Und um Grundfunktionen zu bieten, nutzen wir Object
import Object

### Imports ###

### Klassen ###

class Lokalisierung(Object.Object): 
    def __init__(self, ID=None, art=None,  tag=u"tag:1w6.org,2008:de_DE", kategorie="Text", *args, **kwds): 
        """Lokalisierung des Skriptes"""
        super(Lokalisierung, self).__init__(ID=ID, art=art, tag=tag, kategorie=kategorie, *args, **kwds)
        #: Das Charakterwörterbuch mit allen Charakterdaten aus der Datei. 
        self.locale = self.daten
    
    def basic_data_yaml(self): 
    	return """Name: de_DE
"""
            

### Self-Test ###

def _test(): 
	import doctest
	doctest.testmod()

if __name__ == "__main__": 
   _test() 

if __name__ == '__main__': 
        
        locale = Lokalisierung()
        print locale.locale
        locale.save()

### Self-Test ###



#!/bin/env python
# encoding: utf-8

# Name Generator written by Pointon: 
# http://www.uselesspython.com/showcontent.php?author=27

# Adapted by Arne Babenhauserheide 2007

## I got sick of the absence of plain weird and quirky name generators out there. This comes up with unpronounceable results about 1/5th of the time.
## The namegen function has three parameters: lower_limit and upper_limit govern the length. ratio governs the vowels:consonants ratio. The lower the ratio, the less vowels per consonant.


import random

result = ""
fricatives = ["j", "ch", "h", "s", "sh", "th", "f", "v", "z"]
vowels = ["a", "e", "i", "o", "u", "y", "ya", "ye", "yi", "yo", "yu", "wa", "we", "wi", "wo", "wu", "ae", "au", "ei", "ie", "io", "iu", "ou", "uo", "oi", "oe", "ea"]
consonantsNormal = ["c", "g", "t", "d", "p", "b", "x", "k", "ck", "ch"]
consonantsNasal = ["n", "m", "ng", "nc"]
randomLength = 0
vowelSyllables = 0
vowelRatio = 0

def namegen(ratio=5, lower_limit=3, upper_limit=11):
    ratio = ratio + 3
    if ratio < 4:
        ratio = 4
    elif ratio > 14:
        ratio = 14
    global result, vowelSyllables, randomLength, vowelRatio
    vowelRatio = ratio
    result = ""
    vowelSyllables = 0
    randomLength = random.randrange(lower_limit, upper_limit)
    sylgen()
    return result.capitalize()

def sylgen():
    global result, vowelSyllables, randomLength, syllableOnlyVowels, vowelRatio
    syllableOnlyVowels = 1
    maincongen()
    if vowelSyllables < (vowelRatio/4):
        vowgen()
    maincongen()
    if syllableOnlyVowels == 1:
        vowelSyllables = vowelSyllables + 1
    if len(result) < randomLength:
        sylgen()

def fricgen():
    global result, vowelSyllables, syllableOnlyVowels, fricatives
    result = result + fricatives[random.randrange(0, len(fricatives))]
    syllableOnlyVowels = 0
    vowelSyllables = 0

def vowgen():
    global result, vowels
    result = result + vowels[random.randrange(0, len(vowels))]

def congen():
    global result, vowelSyllables, randomLength, syllableOnlyVowels, consonantsNormal
    result = result + consonantsNormal[random.randrange(0, len(consonantsNormal), 1)]
    syllableOnlyVowels = 0
    vowelSyllables = 0

def con2gen():
    global result, vowelSyllables, syllableOnlyVowels
    if random.randrange(0, 2) == 0:
        result = result + "r"
    else:
        result = result + "l"
    syllableOnlyVowels = 0
    vowelSyllables = 0

def con3gen():
    global result, vowelSyllables, syllableOnlyVowels, consonantsNasal
    result = result + consonantsNasal[random.randrange(0, len(consonantsNasal))]
    syllableOnlyVowels = 0
    vowelSyllables = 0

def maincongen():
    global result, randomLength, vowelRatio
    if len(result) < randomLength:
        randomNumber = random.randrange(0, vowelRatio)
        if randomNumber == 0:
            fricgen()
            if len(result) < randomLength:
                randomNumber = random.randrange(0, vowelRatio/4 * 3)
                if randomNumber == 0:
                    congen()
                    if len(result) < randomLength:
                        randomNumber = random.randrange(0, vowelRatio/2)
                        if randomNumber == 0:
                            con2gen()
                elif randomNumber == 1:
                    con2gen()
                elif randomNumber == 2:
                    con3gen()
        elif randomNumber == 1:
            congen()
            if len(result) < randomLength:
                randomNumber = random.randrange(0, vowelRatio/2)
                if randomNumber == 0:
                    con2gen()
        elif randomNumber == 2:
            con2gen()
        elif randomNumber == 3:
            con3gen()



#!/bin/env python
# encoding: utf-8 

# Schlachtfeld - Großk�mpfe im EWS System 
#   http://rpg-tools-1d6.sf.net
# Copyright © 2007 - 2007 Achim Zien

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA


'''
Characters and methods for use in schlachtfeld module.

Characters act on their own now and can be imported and saved via amov module.

Plans: 
    - Every skill has two assiciated attributes, which increase its value. 

Ideas: 
    - Save everything which happens to each soldier in an attribute of the soldier, so we can track his way through the battle. 
    
        This could be in the style: 
            - Jemor won a battle against Gwarach, but got severly wounded. 
            - Jemor lost the battle against Haske and left the battlefield. 
            - Jemor stayed in the medics tent. 
            - Jemor stayed in the medics tent. 
            - Jemor started into battle again. One of his wounds didn't bother him anymore. 
            - Jemor lost the Battle against Grishkar the beheader and left the battlefield for good. 

Doctests: 
    >>> katt = Char()
    >>> katt.sprache
    'Esperanto'
    >>> katt.value_and_striche_fertigkeiten(striche=11)
    (15, 11)
    >>> katt.value_and_striche_fertigkeiten(striche=4*3)
    (16, 12)
    >>> katt.value_and_striche_fertigkeiten(striche=13)
    (16, 13)
    >>> katt.value_and_striche_fertigkeiten(striche=14)
    (16, 14)
    >>> katt.value_and_striche_fertigkeiten(striche=15)
    (17, 15)
    >>> katt.value_and_striche_fertigkeiten(val=15)
    (15, 9)
    >>> katt.value_and_striche_fertigkeiten(val=18)
    (18, 18)
    >>> katt.amov.eigenschaften
    ' '
    >>> katt.amov.eigenschaften = {"Eleganz": {"Zahlenwert": 12}}
    >>> katt.amov.eigenschaften
    {'Eleganz': {'Zahlenwert': 12}}
    >>> katt.amov.fertigkeiten
    {'Nahkampf': {'Grundwert': 12, 'Striche': 3, 'Zahlenwert': 12}}

'''


#### IMPORTS ####

# for the fighting
from random import randrange as rnd
from random import shuffle
from random import random as rnd01
from random import choice
import math

# For storing values of dicts in other dicts
from copy import deepcopy

# For selection of armies through command line arguments: 
import sys

# EWS standard dice
import pmw6

# Name-Generator
import namen

# Obtain Charakters from yaml-files
from amov import Charakter, Versionsverwaltung
from amov.Object import _
# replaced the alias with the real directory. 

#### IMPORTS ####




#### Top-Level Objects ####

verbose = False #: Print output for debugging?

name_objekt = namen.Name()

# scatter for random attack values
atscatter = [3,2,1]

#### Top-Level Objects ####




#### WRAPPERS ####

def ews():
    """Do one roll with the plusminus d6 die (wrapper for a function in pmw6)."""
    return pmw6.pmw6()

def check(skill,MW):
    """Check if a given skill test reached a min-value (wrapper for a function in pmw6)."""
    return pmw6.check(skill,MW)

def ocheck(skill,MW):
    """Return the degree of success / failure for a skill check (wrapper for a function in pmw6)."""
    return pmw6.open_check(skill,MW)

def ocheck_and_res(skill,  MW):
    """Return the degree of success / failure for a skill check plus the reached value (wrapper for a function in pmw6)."""
    return pmw6.open_check_with_result(skill,MW)


def name_it(language=_("Esperanto")):
    '''Return a name corresponding to the characters language.'''
    return name_objekt.erzeuge(art=language) # ToDo: Fluszh the list of pregenerated names, when the language type changes. 
    
#### WRAPPERS ####




#### CLASSES ####

class Char(object):
    """A single soldier. Will be imported from either a prefab character file from amov or via a template
    
    *args and **kwds makes sure, that we aren't confused, when we get more arguments than we expected."""
    def __init__(self,source="tag:1w6.org,2008:Human",template=True,  *args,  **kwds):
        '''Basic creation of a single Char.'''
        # First call super, so that miltiple inheritance with char as first class can work. 
        super(Char, self).__init__(*args,  **kwds)
        
        # creation from amov template/char
        self.source = source
        self.ID = Versionsverwaltung.Versionen(self.source, _(u"Vorlagen")).neuste
        self.amov = Charakter.Charakter(self.ID)
        
        # rank
        self.name_me(template)
        self.description = self.amov.beschreibung
        self.sprache = self.amov.sprache
        
        # amov combat values
        # When we create him/her, the char is active
        self.active = True
        # and alive
        self.alive = True
        #: The damage a character can take before going down. 
        self.bTP = self.amov.kampfwerte[_('Trefferpunkte')]
        #: The wound value determines, how much damage suffices to inflict a wound on the character.  It's a List containing the value for deep wounds and for critical wounds. A deep wounds weakens the character (3 points off any skill and attribute), a critical wounds disables (the character has to roll, if he keeps standing). 
        self.WS = [self.amov.kampfwerte[_('Wundschwelle')],self.amov.kampfwerte[_('Wundschwelle')] * 3]
        #: The damage which te characters main weapon inflicts. 
        self.dam = self.amov.kampfwerte[_('Hauptwaffe')][_('Waffe')][_('Schaden')]
        #: The attack value of the character. 
        self.attack = self.amov.kampfwerte[_('Hauptwaffe')][_('Kampffertigkeit')][_('Zahlenwert')]
        #self.skill = self.amov.kampfwerte[_('Hauptwaffe')][_('Kampffertigkeit')][_('Name')]
        #: The armor value of the character. 
        self.arm = self.amov.schutz
        
        # variable schlachtfeld values
        self.TP = self.amov.kampfwerte.setdefault(_('Aktuelle Trefferpunkte'), self.bTP) #: The current Health Points of the Char. 
        #: An int, how often the char already managed to walk on, thought the TP were below 0. 
        self.number_of_ko_checks = 0
        self.active = True #: Shows if the the Char is active? 
        # Make sure that we have a wounds dict
        self.amov.kampfwerte.setdefault(_("Wunden"), {})
        # Get the physical wounds, if None are set, they are 0 each. 
        self.wounds = [self.amov.kampfwerte[_("Wunden")].setdefault(_(u'körperlich tief'), 0), self.amov.kampfwerte[_("Wunden")].setdefault(_(u'körperlich kritisch'), 0)] #: deep wounds, critical wounds
        # print self.wounds
        
        # Get the experience, either from the char, if it has it, or calculate it from all other values.
        self.exp = self.amov.ladeGrunddaten().setdefault(_("Striche"), self.calculate_might()) #: The experience of the char. 
        
        #: The attributes of the Char. # TODO: If Eigenschaften changes from ' ' to a dict, update attribute "attributes"
        self.attributes = self.amov.eigenschaften
        
        #: The skills of the char
        self.skills = self.amov.fertigkeiten
        
        #: The equipment of the char. 
        self.equipment = self.amov.ausruestung
        
        #: The main weapon of the char
        self.weapon = self.amov.kampfwerte[_("Hauptwaffe")][_("Waffe")]
        
        #: The fighting skill of the char
        self.fight_skill = self.amov.kampfwerte[_("Hauptwaffe")][_("Kampffertigkeit")]
        
        #: The current armor of the char
        self.armor = self.amov.kampfwerte[_(u"Hauptrüstung")]
        
        
        # Get the morale
        
        if self.amov.eigenschaften == " " or self.amov.eigenschaften.setdefault(_("Morale"), {_("Zahlenwert"): 12})[_("Zahlenwert")] == 12: 
          self.morale = [12]
        else: 
          self.morale_dict = self.amov.eigenschaften.setdefault(_("Morale"), {})
          self.morale = [self.morale_dict.setdefault(_("Zahlenwert"), 12)]


    def save_data_into_char_dict(self): 
        """Save the data from the attributes back into the amov data. 
        
        This means copying all string attributes back into the attributes of the amov object. """
        # First transfer the ID which identifies the char back into the amov data
        self.amov.ID = self.ID
        # Then the chars language. 
        self.amov.sprache = self.sprache
        # Then its name
        self.amov.name = self.name
        # then its description
        self.amov.beschreibung = self.description
        
        # Now on to the battle values
        # Then its basic TP
        self.amov.kampfwerte[_('Trefferpunkte')] = self.bTP
        # And its wound treshold
        self.amov.kampfwerte[_('Wundschwelle')] = self.WS[0]
        # After that the damage of its weapon
        self.amov.kampfwerte[_('Hauptwaffe')][_('Waffe')][_('Schaden')] = self.dam
        # And the attack skill 
        self.amov.kampfwerte[_('Hauptwaffe')][_('Kampffertigkeit')][_('Zahlenwert')] = self.attack
        # Also the armor
        self.amov.schutz = self.arm
        # And the current TP
        self.amov.kampfwerte[_('Aktuelle Trefferpunkte')] = self.TP
        # Also the wounds, deep and critical
        self.amov.kampfwerte[_('Wunden')][_(u"körperlich tief")] = int(self.wounds[0])
        self.amov.kampfwerte[_('Wunden')][_(u"körperlich kritisch")] = int(self.wounds[1])
        # Save experience
        self.amov.ladeGrunddaten()[_("Striche")] = self.exp
        
    
    def save(self): 
        """Save the char into a file, incrementing the minor version, if there is already an existing one."""
        # First prepare the amov data
        self.save_data_into_char_dict()
        # Now call the character object to save itself
        self.amov.save(name=self.name)

    def __str__(self):
        ''' Hopefully nice printout for a soldier.'''
        if self.name is not None: 
            scribble = _('Name: ') + self.name
        else: 
            scribble = _("Unbekannter Charakter")
        scribble += '\n' + _('Sprache: ') + self.sprache
        return scribble
    
    def __lt__(self,  other):
        """Return, if the char is weaker than another char.
        
        We need to do this, because we don't want two chars to be treated as equal, just because they have the same exp (for example they would be seen as duplicates). """
        return (self.exp < other.exp)
    
    def __gt__(self,  other):
        """Return, if the char is stronger than another char.
        
        We need to do this, because we don't want two chars to be treated as equal, just because they have the same exp (for example they would be seen as duplicates). """
        return (self.exp > other.exp)
        

    def name_me(self, template):
        '''Give the Char a random name or import it from the file.'''
        if template:
            self.nametype = self.amov.sprache
            self.name = name_it(language=self.nametype)
        else:
            self.name = self.amov.name
        return


    def damage(self, tp=None, ws=None, hws=None):
        '''Damage a character and check for suvivial. Arguments: hitpoints, wounds, heavy wounds.'''
        # If we're given tp, we work directly with them. 
        if tp is not None and ws is None and hws is None: 
            # First substract the damage from the TP
            self.TP -= tp
            # Then check, if the tp suffice to cause a wound, but not a critical wound
            if self.WS[0] <= tp < self.WS[1]: 
                # And if it does, increase the number of wounds by one. 
                self.wounds[0] += 1
                # At last we check, if the char's still alive. 
                self.checkalive()
                # return a tuple with the new wound and jump out: (deep, critical)
                return 1, 0
            # If it suffices to do a critical wound, ouch! 
            elif self.WS[1] <= tp: 
                # You're out! (at least almost...)
                # Add a critical wound. 
                self.wounds[1] += 1
                # At last we check, if the char's still alive. 
                self.checkalive()
                # return a tuple with the new wound and jump out: (deep, critical)
                return 0, 1
            # If we get to this place, no wound was taken
            return 0, 0
            
        # If we get passed at least a wound, we first reduce the TP
        self.TP -= tp + (ws * self.WS[0]) + (hws * self.WS[1]) # self.WS is a list containing the wound value and the heavy wound value of the character. 
        
        # And then apply the taken wounds
        self.wounds[0] += ws
        self.wounds[1] += hws
        # At last we check, if the char's still alive. 
        self.checkalive()
        return


    def checkalive(self, base_att=None):
        '''Check if a character is still alive after being damaged.
        
        If no base_att is given, this uses the morale. 
        
        To see if a char is still active, we first look, if its TP are <= 0. 
        If they are we do one check against 12, to see if its still active. 
        
        After that we only do one check each time the TP fall to one (further) critical wound value below zero. 
        
        If the TP fall below (or to) 4 critical wound values below 0, the char is definitely dead. 
        
        TODO: Automatically use the correct base_att, for example constitution, or some alias inside the battle_values of the char. 
        
        '''
        
        # If no base att is given, we use the morale. 
        if base_att is None: 
            value = sum(self.morale)
        # else we use the given base_att
        else: 
            value = base_att
        
        # check, if the char is still able to fight. 
        # morale effects: for each morale over 15 > negative TP; for every 6 morale over 12 > max hwounds+1; for every 5 morale over 15 > max wounds+1
        if self.TP <= -self.number_of_ko_checks * self.WS[1]: 
            # First increase the number of ko checks, since this is one :) 
            self.number_of_ko_checks += 1
            # Do a check _without_ wound modifiers, that means a basic ews check. 
            # If the treshold of 4 checks is reached (that means, this is the 5th check), the char definitely goes down. 
            if self.number_of_ko_checks >= 5 or self.TP < -4*self.WS[1] or self.wounds[0] + 3*self.wounds[1] > 18: 
                # If he exceeded 5 KO checks, or his TP fall below 4 times the base attribute, or he got more than 18 wounds (same as TP too low), he's out and will die (see the call to die() below). 
                self.active = False
            else:
                # else it checks, if it can still stand. 
		# The first two KO checks are against 12, the others get consecutively harder.
		if self.number_of_ko_checks == 0:
	                self.active = check(value, MW=12)
		else: 
                	self.active = check(value, MW=(12 + 3*(self.number_of_ko_checks -1)))
			# these are deadly if missed.
			if not self.active:
				self.die()
        
        # In any case, if the TP fall below 4 times the base attribute, or he got more than 18 wounds, he dies. 
        if self.TP < -4*self.WS[1] or (self.wounds[0] + 3*self.wounds[1]) > 18: 
             self.active = False
             self.die()
        return
    
    
    def die(self):
        """Let the Character die."""
        # Don't fight anymore
        self.active = False
        # And don't breathe anymore :) 
        self.alive = False
        
    def check(self, value, mods=[], MW=9):
        '''Make a check with mods on any skill, atttribute or similar. Return if the check was made (True or False). 
        
        Wounds reduce the result. 
        '''
        # mods is a list modifiers which affect the attack roll. 
        return check(value + sum(mods) - 3*(self.wounds[0]+2*self.wounds[1]), MW=MW)
        
    def roll(self, value, mods=[]):
        '''Make a check with mods on any skill, atttribute or similar. Return the absolute result of the roll. 
        
        Wounds reduce the result. 
        '''
        # mods is a list modifiers which affect the attack roll. 
        return value + sum(mods) + ews() - 3*(self.wounds[0]+2*self.wounds[1])

    
    def check_skill(self, skill_name, MW=9, related_skills=[], related_attributes=[], mods=[]):
        """Do a skill check against a target number."""
        # If the char has skills and the skill exists, do a check on it. 
        if self.amov.fertigkeiten != " " and skill_name in self.amov.fertigkeiten.keys(): 
            return self.check(self.amov.fertigkeiten[skill_name][_("Zahlenwert")], mods=mods, MW=MW)
        # If the skill doesn't exist in Fertigkeiten, see if we can do the check on some related skill, but with a malus of 3. 
        # TODO: Implement Jobs and background. 
        elif self.amov.fertigkeiten != " ": 
            for i in related_skills: 
                if i in self.amov.fertigkeiten.keys(): 
                    return self.check(self.amov.fertigkeiten[i][_("Zahlenwert")] - 3, mods=mods, MW=MW)
        # If the char has no skills, see if we can check against a related attribute, but with a malus of 9. 
        if self.amov.eigenschaften != " ": 
            for i in related_attributes: 
                if i in self.amov.eigenschaften.keys() and self.amov.eigenschaften[i][_("Zahlenwert")] >= 12: 
                    return self.check(self.amov.eigenschaften[i][_("Zahlenwert")] - 9, mods=mods, MW=MW)
            # If none of the related Attributes fits, just use the default of 12. 
            return self.check(12 - 9, mods=mods, MW=MW)
        # If the char has no attributes or skills, just use the default attribute of 12. 
        else: 
            return self.check(12 - 9, mods=mods, MW=MW)
        # TODO: Move the code for finding the right value into another function like get_effective_skill_value() or so. 
    
    
    def do_attack(self,  mods=[]):
        '''
        Make an attack-roll to be compared to the enemy's roll. Base method to evaluate comabat.
        Contributions: base skill, weapon, armor, +-d6, wounds (deep: each -3, crit: each -6 while in combat), , group situation (strategy & bias), morale bonus /each point over 12)
        '''
        #print self.name, self.host
        # mods is a list modifiers which affect the attack roll. 
        return self.roll(self.attack, mods=mods) + self.dam + self.arm 
        
    def attack_roll(self,  mods=[]):
        '''
        Make an attack-roll to be compared to the enemy's roll. Base method to evaluate comabat.
        Contributions: base skill, weapon, armor, +-d6, wounds (deep: each -3, crit: each -6 while in combat), , group situation (strategy & bias), morale bonus /each point over 12)
        '''
        #print self.name, self.host
        # mods is a list modifiers which affect the attack roll. 
        return self.roll(self.attack, mods=mods)


    def calculate_might(self,  second_run=False):
        # determine the power of the opposing character
        might = 0
        # Add all Striche of the Eigenschaften to the might
        if self.amov.eigenschaften != " ": 
            for i in self.amov.eigenschaften.keys(): 
                might += self.amov.eigenschaften[i].setdefault("Striche",  self.get_necessary_striche_for_obj({i: self.amov.eigenschaften[i]}))
        # Now add all Striche of the Fertigkeiten to the might
        if self.amov.fertigkeiten != " ": 
            for i in self.amov.fertigkeiten.keys(): 
                might += self.amov.fertigkeiten[i].setdefault("Striche",  self.get_necessary_striche_for_obj({i: self.amov.fertigkeiten[i]}))
        self.exp = might
        if might == 0: 
            if second_run: 
                print self.amov.eigenschaften,  self.amov.fertigkeiten
                raise ValueError("Char has no experience and can't do anything!")
            self.upgrade(0)
            return self.calculate_might(second_run=True)
        else: 
            return might
        
    
    def upgrade(self,expadd,object=('weighted',[])):
        '''
        Randomly enhance the characters skills and attributes. 
        
        Dashes are spent and collected until a higher level is reached. Higher attributes and skills are preferred, thus specializing the character.
        expadd: number of dashes (float - subdashes included)
        object: object of enhancement; 'attribute', 'skill', 'random' or 'weighted' + name of object; standard is 'random'
        
        How to call: 
            >>> char = Char()
            # Upgrade a random value
            >>> char.upgrade(3, object=('random', 0))
            # Upgrade a specific skill (cooking)
            >>> char.upgrade(3, object=("skill", "cooking"))'
            # Upgrade a specific attribute
            >>> char.upgrade(3, object=("attribute", "sensitivity"))
            # Upgrade weighted with a list of things to upgrade.
            >>> char.upgrade(3, object=("weighted", [("attribute",  "sensitivity", 1), ("skill", "cooking", 2)]) # [(cathegory, name, weight), (...)]
        
        Plans: Call the function with a list of Abilities and Attributes, which get improved in the situation in which the Char got the dashes. 
        '''
        # select proper attribute / skill
        if object[0] == 'random':
            listing = []
            # If the char already has Eigenschaften, use a listing of Eigeschaften and Fertigkaietn as base for improvement. 
            if self.amov.eigenschaften != " " and self.amov.fertigkeiten != " ": 
                #Add all Eigenschaften and Fertigkeiten to the listing. 
                for i in self.amov.fertigkeiten.keys(): 
                    listing.append({i: self.amov.fertigkeiten[i]})
                # Now add all Fertigkeiten to the listing. 
                for i in self.amov.eigenschaften.keys(): 
                    listing.append({i: self.amov.eigenschaften[i]})
            elif self.amov.fertigkeiten != " ": # Just use the list of existing Fertigkeiten. 
                for i in self.amov.fertigkeiten.keys(): 
                    listing.append({i: self.amov.fertigkeiten[i]})
            elif self.amov.eigenschaften != " ": # Just use the list of existing Eigenschaften. 
                for i in self.amov.eigenschaften.keys(): 
                    listing.append({i: self.amov.eigenschaften[i]})
            else: 
                raise ValueError("Char has no Eigenschaften an no Fertigkeiten!")
                
            if len(listing) > 1: 
                obj = choice(listing)
            else: obj = listing[0]
            
        elif object[0] in [_('attribute'), _('skill')]:
            if object[0] == _('skill') and self.amov.fertigkeiten != " ":
                listing = self.amov.fertigkeiten
            elif object[0] == _('attribute') and self.amov.eigenschaften != " ":
                listing = self.amov.eigenschaften
            else: listing = None
            # If the object is part of the Fertigkeiten or Eigenschaften, 
            # Set obj as the subtree of the listing dictrionary. 
            if listing is not None and object[1] in listing.keys():
                obj = {object: listing[object[1]]}
            else:
                # add a new skill or attribute
                # For this we add a new key to the respective list (eigenschaften or fertigkeiten)
                # And map that key an an empty dictionary. The base values will be added later on. 
                if object[0] == _("skill"): 
                    # First make sure that the skills are a dict. 
                    if self.amov.fertigkeiten == " ": 
                        self.amov.fertigkeiten = {}
                    # Then add object 1. 
                    self.amov.fertigkeiten.setdefault(object[1], {})
                    obj = {object: self.amov.fertigkeiten[object[1]] }
                elif object[0] == _("attribute"): 
                    # First make sure attributes are a dict. 
                    if self.amov.eigenschaften == " ": 
                        self.amov.eigenschaften = {}
                    # then add object 1. 
                    self.amov.eigenschaften.setdefault(object[1], {})
                    obj = {object: self.amov.eigenschaften[object[1]] }
                if verbose: # Provide some feedback: What kind of value was added, and which one. 
                    print 'Added',  object[0],  object[1]

                
        elif object[0] == 'weighted':
            #: The weightlist contains tuples with the name and the weight of each to-be-increased value.
            weightlist = []
            #: The valuelist contains the possible objects, without weighting. object: {(cathegory, name): {}} - cathegory is 'skill' or 'attribute'
            valuelist = []
            # Add all avaible Eigenschaften and Fertigkeiten as sub-dictionaries to the valuelist. 
            if self.amov.eigenschaften != " " and self.amov.fertigkeiten != " ": 
                for i in self.amov.fertigkeiten.keys(): 
                    valuelist.append({(_("skill"), i ): self.amov.fertigkeiten[i]})
                for i in self.amov.eigenschaften.keys(): 
                    valuelist.append({(_("attribute"), i ): self.amov.eigenschaften[i]})
            elif self.amov.fertigkeiten != " ": # Just use the existing Fertigkeiten. 
                for i in self.amov.fertigkeiten.keys(): 
                    valuelist.append({(_("skill"), i ): self.amov.fertigkeiten[i]})
            elif self.amov.eigenschaften != " ": # Just use the existing Eigenschaften. 
                for i in self.amov.eigenschaften.keys(): 
                    valuelist.append({(_("skill"), i): self.amov.eigenschaften[i]})
            else: 
                raise ValueError("Char has no Eigenschaften and no Fertigkeiten!")
            # Get the biasing for each value by determining how many striche have already been spent without increasing it (+1). 
            # This gives a bias towards advancing higher values. 
            for i in valuelist: 
                for j in i.keys():
                    #: The lowest number of Strichen needed to reach this value. 
                    necessary_striche = self.get_necessary_striche_for_obj(i)
                    #: Biasing determines, how much weight this value gets. 
                    biasing = 1 + i[j].get(_('Striche'),  necessary_striche) - necessary_striche
                    if biasing < 1: 
                        print "Biasing < 1 ! ",  i,  "necessary_striche:",  necessary_striche
                        raise ValueError("Biasing < 1 ! " + str(i) + " necessary_striche: " + str(necessary_striche))
                    weightlist.append((j,  biasing))
            
            # Also add the given list of values to improve. 
            for i in object[1]: 
                weightlist.append(((i[0], i[1]), i[2]))
            
            problist = [] #: A list containing as many instances of each value as it's weight states. 
            for i in range(len(weightlist)):
                for j in range(int(weightlist[i][1])):
                    # Append the name of the item as often as its weight states
                    problist.append(weightlist[i][0]) # The name of the item
            # Determine the weight of all objects added together
            weights_added_together = len(problist)
            dice = rnd01() * weights_added_together
            # in some corner cases a float can be converted to an int with the upper boundary of the float, 
            # In this case weights_added_together. We have to avoid that cornercase. 
            # Example of that cornercase: int(0.99999999999999999) == 1
            while int(dice) == weights_added_together: # If those were equal, it would select the item at position len(problist), which is out of range. 
                dice = rnd01() * weights_added_together
            selected = problist[int(dice)]
            #: The object contains the ability to be raised, only its content, not its name. 
            for i in valuelist: 
                if i.has_key(selected): 
                    obj = i
            # Check if object is defined. 
            try: 
                a = obj
            # If not, it is a new value. 
            except: 
                    # Selected value is not in  the valuelist, so it's a skill or attribute the char doesn't yet have. 
                    # If it's a skill, add it to the skills
                    if selected[0] == _("skill"): 
                        # If we don't yet have skills, make skills a dict. 
                        if self.amov.fertigkeiten == " ": 
                            self.amov.fertigkeiten = {}
                        # Now add the skill
                        self.amov.fertigkeiten[selected[1]] = {}
                        # And set it as the object to raise
                        obj = {selected: self.amov.fertigkeiten[selected[1]]}
                    # If it's an attribute, add it. 
                    if selected[0] == _("attribute"): 
                        # If we don't yet have attributes, make attributes a dict. 
                        if self.amov.eigenschaften == " ": 
                            self.amov.eigenschaften = {}
                        # Now add the attribute
                        self.amov.eigenschaften[selected[1]] = {}
                        # And set it as the object to raise. 
                        obj = {selected: self.amov.eigenschaften[selected[1]]}
                    
        # Now we know which ability or attribute we want to raise. Say so: 
        if verbose: 
            print self.name, "raised",  obj.keys()[0],  "with",  expadd,  "Strichen", 
        # And save it for later use.
        obj_before = deepcopy(obj)
        
        # As next step, we need to determine, if it gets raised, and by how much. 
        
        # First get the value of the selected object. 
        # If it's a Fertigkeit, get the Grundwert. 
        if self.amov.fertigkeiten != " ": 
            for i in self.amov.fertigkeiten.keys(): 
                # Check if it has a skill key i
                if obj.has_key((_("skill"), i)): 
                    sub_obj = obj[(_("skill"), i)]
                    val = sub_obj.setdefault(_("Grundwert"),  sub_obj.setdefault(_("Zahlenwert"),  3))
                    # all skills / attributes should have a 'Striche' item
                    striche_old = sub_obj.setdefault(_("Striche"),  self.value_and_striche_fertigkeiten(val=sub_obj.setdefault(_("Grundwert"),  3))[1])
                    sub_obj.setdefault(_("Striche"),  striche_old)
                    # add xp, if applicable raise skill / attribute
                    sub_obj[_('Striche')] += expadd
                    sub_obj[_("Grundwert")] = self.value_and_striche_fertigkeiten(striche=sub_obj[_('Striche')] )[0]
                    if sub_obj[_("Zahlenwert")] < sub_obj[_("Grundwert")] : 
                        sub_obj[_("Zahlenwert")] = sub_obj[_("Grundwert")] #TODO: We need a character method, which gets the Zahlenwert from the Grundwert. 
    
        # If it's an attribute, get the Zahlenwert. 
        if self.amov.eigenschaften != " ": 
            for i in self.amov.eigenschaften.keys(): 
                if obj.has_key((_("attribute"), i)): 
                    sub_obj = obj[(_("attribute"), i)]
                    striche_old = sub_obj.setdefault(_("Striche"),  self.value_and_striche_eigenschaften(val=sub_obj.setdefault(_("Zahlenwert"),  12))[1])
                    sub_obj[_("Striche")] = striche_old + expadd
                    sub_obj[_("Zahlenwert")] = self.value_and_striche_eigenschaften(striche=sub_obj[_("Striche")])[0]
        
        if verbose: 
            print "to",  obj # finishing upgrade line of output
        self.attack = self.amov.kampfwerte[_('Hauptwaffe')][_('Kampffertigkeit')][_('Zahlenwert')]
        return obj_before, obj
    
    
    def get_necessary_striche_for_obj(self,  obj):
        necessary_striche = 0
        if self.amov.fertigkeiten != " ": 
            for i in self.amov.fertigkeiten.keys(): 
                if obj.has_key(i): 
                    # Every Fertigkeit needs a Grundwert
                    obj = obj[i]
                    val = obj.setdefault(_("Grundwert"),  obj.setdefault(_("Zahlenwert"),  3))
                    # all skills / attributes should have a 'Striche' item
                    striche = obj.setdefault(_("Striche"),  self.value_and_striche_fertigkeiten(val=obj.setdefault(_("Grundwert"),  3))[1])
                    necessary_striche = self.value_and_striche_fertigkeiten(val=obj.setdefault(_("Grundwert"),  3))[1]
        if self.amov.eigenschaften != " ": 
            for i in self.amov.eigenschaften.keys(): 
                if obj.has_key(i): 
                    obj = obj[i]
                    striche = obj.setdefault(_("Striche"),  self.value_and_striche_eigenschaften(val=obj.setdefault(_("Zahlenwert"),  12))[1])
                    necessary_striche = self.value_and_striche_eigenschaften(val=obj.setdefault(_("Zahlenwert"),  12))[1]
        return necessary_striche
    
    
    def value_and_striche_fertigkeiten(self,  val=None,  striche=None):
        """Return value and striche"""
        striche_fertigkeiten = [3,  6, 9, 12]
        # First we do the case, when we get striche and want a value
        # For a low number of striche, this is quite easy. 
        if striche is not None and striche <= 3: 
            val = striche_fertigkeiten[int(striche)]
            return val,  striche
        # for higher numbers of strichen, itthe necessary list is a bit longer. 
        elif striche is not None and striche > 3: 
            # first create an aggregator
            necessary_striche = 3
            striche_per_point = 2
            effective_value = 12
            # We raise the value until the necessary_striche is higher of equal to the striche. 
            # If it's higher, we reduce the value afterwards. 
            # add all values together, up to the number of strichen. 
            while necessary_striche < striche: 
                effective_value += 1
                necessary_striche += int(striche_per_point + 0.1) # this +0.1 is necessary to avoid some crazy rounding down at 2.0
                striche_per_point += 1/3.0
            if necessary_striche > striche: 
                effective_value -= 1
                necessary_striche -= int(striche_per_point)
            return effective_value,  striche
        
        # If we have a value, we return the number of necessary_striche, 
        # If teh value is lower than or equal to 12, it's easy to get the striche 
        # by inverse searching in the striche_fertigkeiten list. 
        if val is not None and val <= 12: 
            necessary_striche = striche_fertigkeiten.index(val)
            return val,  necessary_striche 
        # If the value is higehr than 12, we increase the effective_value in steps, 
        # until we reach the given value, increasing the striche as by the 1d6 vobsy rules. 
        elif val is not None and val > 12: 
            necessary_striche = 3
            striche_per_point = 2
            effective_value = 12
            while effective_value < val: 
                effective_value += 1
                necessary_striche += int(striche_per_point + 0.1) # this +0.1 is necessary to avoid some crazy rounding down at 2.0
                striche_per_point += 1/3.0
            return effective_value,  necessary_striche    
    
    
    def value_and_striche_eigenschaften(self,  val=None,  striche=None):
        """Return value and striche."""
        vorzeichen = 1
        if striche is not None and striche < 0: 
            striche = -striche
            vorzeichen = -1
        if striche is not None and striche >= 0: 
            # first create an aggregator
            necessary_striche = 0
            striche_per_point = 1
            effective_value = 12
            # We raise the value until the necessary_striche is higher of equal to the striche. 
            # If it's higher, we reduce the value afterwards. 
            # add all values together, up to the number of strichen. 
            while necessary_striche < striche: 
                effective_value += 1
                necessary_striche += int(striche_per_point + 0.1) # this +0.1 is necessary to avoid some crazy rounding down at 2.0
                striche_per_point += 1/3.0
            if necessary_striche > striche: 
                effective_value -= 1
                necessary_striche -= int(striche_per_point)
            if vorzeichen == 1: 
                return effective_value,  striche
            else: 
                return 24 - effective_value,  -striche
        
        if val is not None and val < 12: 
            val = 24 - val
            vorzeichen = -1
        # If the value is higehr than 12, we increase the effective_value in steps, 
        # until we reach the given value, increasing the striche as by the 1d6 vobsy rules. 
        if val is not None and val >= 12: 
            necessary_striche = 0
            striche_per_point = 1
            effective_value = 12
            while effective_value < val: 
                effective_value += 1
                necessary_striche += int(striche_per_point + 0.1) # this +0.1 is necessary to avoid some crazy rounding down at 2.0
                striche_per_point += 1/3.0
            if vorzeichen == 1: 
                return effective_value,  necessary_striche
            else: 
                return 24 - effective_value,  -necessary_striche
        
        
    def fight_round(self,  other, styles_self=[], styles_other=[]):
        """One battle action against another char.
        
    @param styles_self: The different styles the char should use (defense, pull back, escape, target: area). 
    @type styles_self: A list of Strings. 
    
    @param styles_other: The different styles the enemy uses. 
    @type styles_other: A list of Strings. 
    
    @return: [ If the char won (bool), [your new wounds, critical, taken tp damage], [the 
enemies new wounds, critical, taken tp damage] ]

    Plans: 
        - TODO: Add options for own fight style. 
        - TODO: Add options for other fight style. 
        
    Planned options for own fight style: 
        - defense (+6, no damage enemy)
        - pull back (+9, no damage enemy)
        - try to escape. 
        - target specific body area for additional damage. 
        - Shield/evasion (treshold for the enemies diff, below which no hit was archieved).
    
    Ideas: 
        - Define a damage multiplier, which shows how effective the damage type is against the specific enemy -> do that in the char itself. 
        - Define the effectivity of the armor against the weapon. 
        - Get the treshold directly from the char. 
."""
	# Initialize all variables empty
        won = None #: Did we win this round?
        mods_self = [] #: The modifiers for the protagonists attack roll. 
        mods_other=[] #: The modifiers for the enemies attack roll. 
        deep_wounds_self, deep_wounds_other, critical_wounds_self, critical_wounds_other = 0, 0, 0, 0
        
        
        
        
        #: The damage we get
        self.damage_self = 0
        #: The damage the other gets
        self.damage_other = 0
        
        # Do the rolls for both fighters. 
        attack_self = self.attack_roll(mods=mods_self)
        attack_other = other.attack_roll(mods=mods_other)
        
        # If I throw higher
        if attack_self > attack_other: 
            won = True
            # Now check for damage (we hit)
            # The damage consists of several factors. 
            # First the difference between the attack rolls. 
            self.damage_other += attack_self - attack_other
            # Then the damage of our weapon. 
            self.damage_other += self.dam
            # And substracted the armor of the other. 
            self.damage_other -= other.arm
        
        # Clean out negative damage. If the damage is below zero, the armor caught all damage. 
            if self.damage_other < 0: 
                self.damage_other = 0
            
            # Now actually do the damage. This returns a tuple: (new deep wounds, new critical wounds)
            deep_wounds_other, critical_wounds_other = other.damage(tp=self.damage_other)
        
        # If the other rolls better
        elif attack_self < attack_other: 
            won = False
            # Check for our damage (the other hit)

            # The damage consists of several factors. 
            # First the difference between the attack rolls. 
            self.damage_self += attack_other - attack_self
            # Then the damage of our weapon. 
            self.damage_self += other.dam
            # And substracted the armor of the other. 
            self.damage_self -= self.arm

            # Clean out negative damage. If the damage is below zero, the armor caught all damage. 
            if self.damage_self < 0: 
                self.damage_self = 0
            
            # Now actually take the damage. This returns a tuple: (new deep wounds, new critical wounds)
            deep_wounds_self, critical_wounds_self = self.damage(tp=self.damage_self)

        # If we get the same result, the attacker wins, us. 
        else: 	
            won = True
            # Now check for damage (we hit)
            # The damage consists of several factors. 
            # First the difference between the attack rolls. 
            self.damage_other += attack_self - attack_other
            # Then the damage of our weapon. 
            self.damage_other += self.dam
            # And substracted the armor of the other. 
            self.damage_other -= other.arm
            
            # Clean out negative damage. If the damage is below zero, the armor caught all damage. 
            if self.damage_other < 0: 
                self.damage_other = 0
            
            # Now actually do the damage. This returns a tuple: (new deep wounds, new critical wounds)
            deep_wounds_other, critical_wounds_other = other.damage(tp=self.damage_other)
        	
        return won, [deep_wounds_self, critical_wounds_self, self.damage_self], [deep_wounds_other, critical_wounds_other, self.damage_other]


    def fight_one_roll_battle(self, other):
        """Fight a battle with the one roll system (one roll decides)."""
        
        # attack rolls
        res = self.do_attack() - other.do_attack()
        
        # damage, formatting: (tp, wounds, heavy wounds)
        # style-formatting:  (skill mod, (win own damage[base,stepsize]), (loose own damage[base,stepsize]), special pars
        if res == 0:
            self.damage(0,3,0)
            other.damage(0,3,0)
            if verbose: 
                print "Both are still standing but wounded (or dead) - none won the battle."
            # We don't need to check anything else. 
            return 
        else:
            if res > 0:
                winnerc = self
                loserc = other
            else:
                winnerc = other
                loserc = self
            res = abs(res)
            
            # TODO: Add battle styles again (from BatteField)
            
            # First get the damage for both
            winner = max(3-res/3, 0)
            loser = max(3+res/3, 0)
            if verbose:
                print winnerc.name, "won against", loserc.name
            
            # Then let the winner take damage
            winnerc.damage(0,winner,0)
            if verbose: 
                print winnerc.name, "takes", winner, "wounds." 
            
            # And let the loser take damage    
            loserc.damage(0,loser,0)
            # The loser goes down
            loserc.active = False
            
            if verbose: 
                print loserc.name, "takes", loser, "wounds."
            

    def get_battle_result_values(self): 
        """Return the values which change in battle."""
        return {"TP": self.TP, "Wounds [deep, critical]": self.wounds, "Active": self.active, "Alive": self.alive}
        
    
    def fill_missing_values(self):
        """Fill in missing values in the charsheets. """
        pass # TODO: Move all sheet-cleaning and sheet-filling code into this method. 



#### CLASSES ####

#### Self-Test ####

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__": 
    _test()
    katt = Char(template=False)
    katt.amov.eigenschaften = {"Eleganz": {"Zahlenwert": 12}}
    print "Before Upgrade"
    print katt.amov.fertigkeiten
    print katt.amov.eigenschaften
    print "Add  3 dashes"
    katt.upgrade(3)
    print "After Upgrade"
    print katt.amov.fertigkeiten
    print katt.amov.eigenschaften
    print "Saving char."
    katt.save()
#### Self-Test ####



#!/usr/bin/python
# encoding: utf-8

# Yould - A name generator generator
# This program was initially written by © Yannick Gingras - 2007

# Is was adapted by © Arne Babenhauserheide in 2007. 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

#### Imports ####

from sys import exit as sys_exit
from sys import argv as sys_argv
from random import seed, random
seed() # Seed as the os states, or from time

#### Imports ####

#### Control Parameters #### 

maximum_useful_probability = 0.0025

min_probability = 0.0000000025
max_probability = 0.00025

#### Control Parameters #### 

def get_c(probs, draw):
    tot = 0
    for prob, c in probs:
        tot += prob
        if tot >= draw:
            return c
    return c

def gen3_name(tr3_prob):
    choices = []
    pprev = prev = None
    c = get_c(tr3_prob[(None, None)], random())
    while c != None:
        choices.append(c)
        pprev = prev
        prev = c
        c = get_c(tr3_prob[(pprev, prev)], random())
        
    return "".join(choices)

def name3_prob(tr3_prob, name):
    prob = 1.0
    pprev = prev = None
    for c in name:
        prob *= filter(lambda rec:rec[1] == c,
                       tr3_prob[(pprev, prev)])[0][0]
        pprev = prev
        prev = c
    return prob * filter(lambda rec:rec[1] == None,
                         tr3_prob[(pprev, prev)])[0][0]

def generate_names(tr3_prob, min_l, max_l, nb_w, nb_d, min_prob = min_probability, max_prob = max_probability): 
    
    names = []
    tried = []
    while len(names) < nb_w:
        name = gen3_name(tr3_prob)
        # catch bad set min_prob and max prob
        # Minuímum probaility 
        if min_prob > maximum_useful_probability / 28: 
            min_prob = maximum_useful_probability / 28
        # min_prob needs to be at least factor 28 smaller than max_prob. Experimental result
        if min_prob > max_prob / 28: 
            if max_prob < maximum_useful_probability: # min-prob was badly set
                min_prob = max_prob / 28
            else: # max-prob was badly set.
                max_prob = min_prob * 28
        if min_l <= len(name) <= max_l \
               and name not in tried \
               and name3_prob(tr3_prob, name) > min_prob \
               and name3_prob(tr3_prob, name) < max_prob:
            # print "%.010f: %s" % (name3_prob(tr3_prob, name), name)
            names.append(name)
        if name not in tried: 
            tried.append(name)

    # TODO: fix the scoring 
    #for prob, name in sorted([(name3_prob(tr3_prob, name), name)
    #                          for name in names]):
    #    print "%.010f: %s" % (name3_prob(tr3_prob, name), name)
    return names #, tried

def print_names(names, tried): 
    print "tried", len(tried), "names"
    print ", ".join(sorted(names))
    
#### Self-Test ####

if __name__ == "__main__":
    if len(sys_argv) != 6:
        print "USAGE: %s PROBS MIN_L MAX_L NB_W NB_D" % sys_argv[0]
        print "  PROBS is the language transition table"
        print "  MIN_L is the minimum desired word length"
        print "  MAX_L is the maximum desired word length"
        print "  NB_W  is the number of words to generate"
        print "  NB_D  is the number of free domain required (0 to 4)"
        sys_exit(1)
    tr3_prob = eval(open(sys_argv[1]).read())
    min_l = int(sys_argv[2])
    max_l = int(sys_argv[3])
    nb_w  = int(sys_argv[4])
    nb_d  = int(sys_argv[5])
    names = generate_names(tr3_prob, min_l, max_l, nb_w, nb_d)
    print_names(names, "")

#### Self-Test ####




#!/bin/env python
# This Python file uses the following encoding=utf-8

# Amo - Allgemeine Modulare Objektverwaltung im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

# Dieses Skript übersetzt tag-IDs zu Dateien. 

# Als Eingabe erhalten wir eine tag-ID im Format
# tag:url,yyyy:name
# eine Version und eine Kategorie
# Als Ausgabe wollen wir eine Datei-Adresse im Format
# Kategorie/name-version-yyyy-url.yml

"""Read an input-object ('ID' supplies the tag) and translate it to a file-path"""

import tag_zu_objekt
import tag_zu_datei_von_tag

class Datei: 
        """Read a tag and return a
corresponding file-path"""
        def __init__(self, Eingabe): 
                self.Eingabe = Eingabe
                self.tag_string = self.Eingabe["ID"]
                self.version = self.Eingabe["Version"]
                # wir holen uns das Objekt das dem tag entspricht.
                self.tag_zum_objekt = tag_zu_objekt.tag_zu_objekt(self.tag_string)
                self.tag_objekt = self.tag_zum_objekt.tag_objekt()
        
        def dateiname_ausgeben(self): 
                # Mit dem Tag_objekt und dem Eingabe-Wörterbuch können wir nun den Datei-String erzeugen. 
                tag_zu_dat_v_t = tag_zu_datei_von_tag.Datei(tag=self.tag_string, kategorie=self.kategorie_ausgeben(), version=self.version)
                datei = tag_zu_dat_v_t.dateiname_ausgeben()
                return datei
        
        def kategorie_ausgeben(self):
                return self.Eingabe['Kategorie']
                
        def name_ausgeben(self): 
                return self.tag_objekt['Name']
		
        
#### Self-Test ####

if __name__ == "__main__": 
    pass

#### Self-Test ####



#!/bin/env python
# encoding: utf-8
"""
Read out and write out an Object from/to a yaml-file.

This is a base class which gets used by other classes. 

Just call it with an ID-dict and a basic template (object or dict) and get the object or dict via Objekt.objekt, or write it via Objekt.write(<dict>). 

To tell it NOT to create a file, if it is missing, pass the parameter: erstellen=False
"""


# Charakterverwaltung - Verwalte Charaktere im lesbaren YAML Format
# Copyright © 2007 - 2007 Arne Babenhauserheide

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA 02110-1301 USA

# Diese Datei ist ein Container für die klasse Charaktere

### Imports ###

# Für das Dateiformat brauchen wir yaml, wir brauchen load udn dump
from yaml import load as yaml_load
from yaml import dump as yaml_dump

# Für die Übersetzung von IDs in Dateinamen brauchen wir noch tag_zu_datei

from ein_und_ausgabe import tag_zu_datei

# Um Charaktere per Kommandozeile übergeben zu können brauchen wir sys
import sys

# Für die Prüfung ob Charakterdateien schon existieren brauchen wir außerdem os.path

from os.path import isdir as path_isdir
from os.path import exists as path_exists

from os import makedirs

from os.path import dirname

### Imports ### 

### Classes ###

class Objekt: 
    def __init__(self, ID=yaml_load("""# tag:1w6.org,2008:ID
ID: tag:draketo.de,2007:Sskreszta
Version: 0.15
Kategorie: Charaktere
"""), template=yaml_load("""# tag:1w6.org,2008:Charakter
- Name: " "
- Grunddaten: 
    Beschreibung: " "
    Herkunft: 
        Sprache: ' '
        Region: ' '
    Stimmung: ' '
- Werte:
    - Eigenschaften: " "
    - Fertigkeiten: 
        Nahkampf: &id001
          Zahlenwert: 12
    - Merkmale: " "
- Ausrüstung: 
    Waffen:
      Waffenlos: &id002
        Name: Waffenlos
        Schaden: 1
    Rüstung: 
        Stoffkleidung: &hauptruestung
            Name: Stoffkleidung
            Schutz: 1
- Kampfwerte:  
    Hauptwaffe:
      Kampffertigkeit: *id001
      Waffe: *id002
    Trefferpunkte: 24
    Wundschwelle: 4
    Hauptrüstung: *hauptruestung"""),  erstellen=True): 
        self.ID = ID
        self.versions_tag = "tag:1w6.org,2008:Objekt"
        self.template = template
        self.erstellen = erstellen
        self.template_yaml = yaml_dump(self.template, default_flow_style=False, allow_unicode=True)
        self.objekt = self.laden()
    
    def laden(self):
        """Load an object from a yaml-file."""
        # print "Charakterbogen aus Datei", self.dateiname(), "wird geladen" 
        # Erst sicherstellen, dass alle config-ordner existieren. 
        if not path_isdir(dirname(self.dateiname())) and self.erstellen: 
            makedirs(dirname(self.dateiname()))
        
        if path_exists(self.dateiname()): 
          datei = open(self.dateiname(), "r")
          charakter = yaml_load(datei.read())
          datei.close()
          return charakter
        
        elif self.erstellen: 
          # print "Datei", self.dateiname(), "existiert nicht. Wird erstellt."
          datei = open(self.dateiname(), "w")
          datei.write(self.template_yaml)
          datei.close()
          return self.laden()
        else: 
           raise FileNotFoundException("Die Datei existiert nicht und konnte nicht erstellt werden.")
    
    def dateiname(self): 
        """Return path and filename based on Cathegory, Version and Tag."""
        tag_zu_dat = tag_zu_datei.Datei(self.ID) 
        return tag_zu_dat.dateiname_ausgeben()
    
    def kategorie(self):
        """Return the cathegory of the Object"""
        tag_zu_dat = tag_zu_datei.Datei(self.ID)
        return tag_zu_dat.kategorie_ausgeben()
    
    def write(self, neue_version=False): 
        if not neue_version: 
            # print u"öffne", self.dateiname()
            datei = open(self.dateiname(), "w")
            # print u"Schreibe", self.kategorie(), self.objekt_name()
            datei.write(self.yaml())
            datei.close()
        else: # TODO: Direkt in eine neue Version schreiben ermöglichen. 
            raise NotImplementedException("Direkt in eine neue Version schreiben ist noch nicht implementiert.")
    
    def yaml(self): 
        return "# " + self.versions_tag + "\n" + yaml_dump(self.objekt, default_flow_style=False, allow_unicode=True)
    
    def objekt_name(self): 
        tag_zu_dat = tag_zu_datei.Datei(self.ID)
        return tag_zu_dat.name_ausgeben()

### Classes ###

### Selt-Test ###

if __name__ == '__main__': 
    objekt = Objekt()
    print objekt
    print objekt.yaml()



#!/usr/bin/env python
# encoding: utf-8

###########################################################################
#    Copyright (C) 2007 by Achim Zien und Arne Babenhauserheide                                      
#    <arne_bab@web.de>                                                             
#
# Copyright: See COPYING file that comes with this distribution
#
###########################################################################

"""Calculate a fight between two chars (given as list) according to the EWS One Roll Fight system."""

verbose = False

#### imports ####

#### /imports ####

#### Fonctions ####

def fight(chars):
    """Calculate a fight between two individuals with the One Roll fight system
    
    Plans: 
        - Add Fatigue, since each single fight drains each fighter, so people can't fight forever with their full strength, Fatigue is an attribute. From this we determine a fatigue value (2*Fatigue). If this falls to 1/2, the fighter gets 1 point malus. If it falls to 1/6th, the fighter gets 3 points malus. If it falls to 0, the fighter gets 6 points malus ans has to check, if he stays standing. The he has to check if he stays standing again each time, the fatigue value drops below a multiple of the Fatigue attribute below zero. At fatigue value = -4x Fatigue attribute, the fighter falls unconscious. 
    """
    
    # If we have only two chars, we just let them battle it out
    if len(chars) == 2: 
        chars[0].fight_one_roll_battle(chars[1])
    else: 
        # TODO: Group battle isn't implemented yet
        raise Exception("Mass battle isn't implemented, yet")

    

#### /Functions ####

#### Self-Test ####

if __name__ == "__main__": 
    from schlachtfeld.amov import Charakter, Versionsverwaltung
    from schlachtfeld.Char import *
    verbose = true
    chars = [Char() for i in range(2)]
    #for i in range(2):
        #char = Char()
        #chars.append(char)
    print "Characters:\n", chars
    print "\nStarting the fight:"
    fight(chars)
    

#### /Self-Test ####



#!/usr/bin/env python
# encoding: utf-8

"""The battle module provides functions for battles between more than 2 participants."""

#### CONTROL PARAMETERS ####

# BIAS

# each factor of biasfactor gives +1 to attack for superior numbers
biasfactor = 1.4

# multiplier for the bias calculation
biasmultiplier = 2.0

# the maximum biasfactor (if multiplied with biasmultiplier, approached asymptotically
biascutoff = 8

#### Imports ####

# We need the Math module for the logarithm
from math import log

def bias(size_of_group_0, size_of_group_1):
    """Return the modifier for number of fighters. 

If group 0 is bigger, the modifier is positive, if group 1 is bigger, it is negative.
"""
    if max(size_of_group_0, size_of_group_1) < 5:
        if size_of_group_0 > size_of_group_1: 
            bias = int(round(3 * float(size_of_group_0 - size_of_group_1)/max(1, size_of_group_1)))
        else: 
            bias = (-1)*int(round(3 * float(size_of_group_1 - size_of_group_0)/max(1, size_of_group_0)))
    else: 
        q = log(float(max(1, size_of_group_0))/max(1, size_of_group_1),biasfactor)
        bias = int(round(biasmultiplier * q/(1+abs(q)/biascutoff)))
    return bias



#!/usr/bin/env python
# encoding: utf-8

# EWS - An RPG backend module providing character management and interaction. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

"""The ews ("Ein Wuerfel Sytem" is german for "One Die System") is a free rpg backend which covers char interaction and char management -> http://1w6.org. 

Chars have attributes, abilities, edges/flaws, equipment and battle values (directly derived from the attributes and abilities). 


Usage from command line: 

    - None yet. 


Examples: 

    - None yet. 


Use-Cases: 

 - Creating Chars
    
    Create and print a char from a tag-string
        >>> from char import Char
        >>> tag_string = "tag:1w6.org,2008:Human"
        >>> char_from_tag = Char(source=tag_string, template=False) 
    
    template=False says: Go with the name inside the stored file or (if None is avaible) in the tagstring. 
    
    Else it would create a new name for it. 
    
    TODO: Change the default to template=False (some checking of other libs necessary for that)
        >>> print char_from_tag
        Name: Human
        Sprache: Esperanto
    
    TODO: Create a char from a char dict. 
    
    TODO: Print location of character file. 
    
    
 - Combat
    
    Create two chars and let them fight one round. 
        >>> char1 = Char()
        >>> char2 = Char()
        >>> print char1.wounds, char2.wounds
        [0, 0] [0, 0]
        >>> battle_result = char1.fight_round(char2)
        >>> character_stats = (char1.get_battle_result_values(), char2.get_battle_result_values())  
        >>> # Gives TP and wounds for Char 1 and 2
    
    Let the two chars fight a battle with the one roll battle system. 
        >>> char1.fight_one_roll_battle(char2)
        >>> result = (char1.get_battle_result_values(), char2.get_battle_result_values()) 
        >>> # Dict for char1 and for char 2
    
    TODO: Fight a whole battle with the complex battle system. 
    
    
 - Attributes
    
    List all attributes. If the char has None, print a single space. 
        >>> char1.attributes
        ' '
    
    
 - Skills and checks
    
    List all skills. If the char has None, print a single space. 
        >>> char1.skills
        {'Nahkampf': {'Grundwert': 12, 'Striche': 3, 'Zahlenwert': 12}}
    
    Do a skill test (default target number is 9)
        >>> print "Do we manage to cook a nice meal?"
        Do we manage to cook a nice meal?
        >>> result = char1.check_skill("cook")
    
    Do a skill test against another target number. 
        >>> print "Do we manage to cook an exceptional meal?"
        Do we manage to cook an exceptional meal?
        >>> result = char1.check_skill("cook", MW=18)
    
    TODO: Check how good we manage something. 
    
    TODO: Check if we manage to make a roll and how good we manage it. 
    
    
 - Competition (skill vs. skill and similar)
    
    TODO: Let two chars compete for one round. 
    
    TODO: Let two chars do a full competition. 
    
    
 - Equipment
    
    Get the current equipment of the char. 
        >>> for i in char1.equipment: print char1.equipment[i] 
        {'Stoffkleidung': {'Name': 'Simple garb', 'Schutz': 1}}
        {'Waffenlos': {'Name': 'Iron knuckles', 'Schaden': 1}}
        >>> # TODO: Fix to make it nicer to use. 
    
    
    Get the current combat equipment of the char (armor and weapon). 
        >>> char1.weapon
        {'Name': 'Iron knuckles', 'Schaden': 1}
        >>> char1.armor
        {'Name': 'Simple garb', 'Schutz': 1}
    
    TODO: Change armor and weapon (and weapon skill). 
    
    TODO: Get the current clothes of the char. 
    
    TODO: Change the current clothes of the char. 
    
    
 - Improving Chars
    
    Improve a char by a 3 points at random (about the value to get for one gaming session in a hreo setting). 
        >>> old_and_new_data = char1.upgrade(3)
    
    Upgrade with a weighted list of attributes and skills which could be improved additionally to known skills and attributes. 
        >>> old_and_new_data = char1.upgrade(3, object=("weighted", [("attribute",  "sensitivity", 1), ("skill", "cooking", 2)]))
    
    Upgrade a specific skill or attribute. 
        >>> old_and_new_data = char1.upgrade(3, object=("skill", "talking senslessly"))
        
    # TODO: Add attibute explicitely. 
    
    
 - Saving Chars
    
    Save the changed (wounded but improved) Char as new template. 
    
    Commented out, because this creates new files. 
        >>> # char1.name = char1.amov.tagname
        >>> # char1.save()
    
    Save the changed Char as a new char and get the new tag to call it again. 
    
    Commented out, because this creates new files. 
        >>> # tagstring = char1.amov.tagstring_without_name + char1.name # The tag to call the char
        >>> # char1.save()
        >>> # char1_again = Char(source=tagstring, template=False)
    
    
 - Finishing Chars off :) 
    
    And in the end: Die 
        >>> char1.die()


Plans: 
    - Change char file format: an empty dict shoulnd't be represented by " " anymore, but by {} (the real yaml represenation of an empty dictionary). 


Source URL (Mercurial): U{http://rpg-1d6.sf.net/hg/1d6}

PyPI URL: U{http://pypi.python.org/pypi/ews}
"""

#### Package information ###

__version__ = "0.1"

#### Package information ###

#### Imports ####

from char import Char

#### Imports ####

#### Self-Test ####

def _test():
    """Launch the doctests."""
    from doctest import testmod
    testmod()

if __name__ == "__main__": 
    _test()



# Context-free grammar random name generator
# Jeremy Thurgood <firxen@gmail.com>
# Highly experimental at present, but sort of working

import random
import re
import sys
import yaml


class GrammarError(RuntimeError):
    pass

class CFNameGen(object):
    # This should be done using gettext for i18n, but I can't be bothered to figure
    # out how to do it properly, so I'm using replacement strings for now.
    stringUndefinedNonTerminal = "Undefined non-terminal \"%(undefinedNonTerminal)s\" in rule \"%(rule)s\"."

    # Regular expression to catch non-terminals, used frequently, so global
    reNonTerminal = re.compile(r"<(\w+)>")

    def __init__(self, nameGrammar):
        """Create a namegen object.

        We take a grammar dict, as before the Great Refactoring.
        """

        self.checkTypes(nameGrammar)
        self.grammar = nameGrammar

    # checkTypes() is only useful while testing with internally specified grammars.
    # Once we're parsing an external file it becomes unnecessary since we generate
    # the data types ourselves instead of asking a human to do it.  As such, error
    # strings are hardcoded.  Anyone who sees them would be messing around in here
    # anyway.
    def checkTypes(self, nameGrammar):
        """Check given grammar object for correct datatypes.
        
        This function is only really necessary while the grammar's still being
        specified in here.  It will likely disappear when we parse the grammar from a
        data file.
        """
        if not isinstance(nameGrammar, dict):
            raise GrammarError("Grammar data is not a dictionary!")
        for rule, rhs in nameGrammar.items():
            if not isinstance(rhs, list):
                raise GrammarError("Rule \"%s\" is not a list!" % rule)
            for option in rhs:
                if not isinstance(option, str):
                    raise GrammarError("Rule \"%s\" does not contain only strings!" % rule)
                
    # Grammar verification stuff follows.  We can probably make this throw warnings
    # and correct problems, but that's a job for another day.  Incorrect grammars
    # probably won't provide useful output anyway.  If this stuff gets big enough
    # it may be pushed into its own module.

    def checkUndefinedNonTerminals(self, nameGrammar):
        """Check given grammar for undefined non-terminals.
        
        An undefined non-terminal is a non-terminal symbol used in a symbol
        definition that has no definition of its own and cannot therefore be
        expanded.  Undefined non-terminals can lead to ugly error messages
        instead of beautifully generated names.
        """
        for rule, rhs in nameGrammar.items():
            for option in rhs:
                tempStr = option
                matchNonTerminal = self.reNonTerminal.search(tempStr)
                while matchNonTerminal:
                    if matchNonTerminal.group(1) not in nameGrammar:
                        return {"undefinedNonTerminal": matchNonTerminal.group(1), "rule": rule}
                    tempStr = self.reNonTerminal.sub("", tempStr, 1)
                    matchNonTerminal = self.reNonTerminal.search(tempStr)

    def checkUnproductiveNonTerminals(self, nameGrammar):
        """Check grammar for possibly unproductive non-terminals.
        
        An unproductive non-terminal is a non-terminal symbol that cannot be
        converted to a terminal symbol in the given grammar.  A good example of this
        is a non-terminal symbol that includes itself in its definition.
        
        This function is currently very basic and should be extended (rewritten?) to
        allow warnings for _possible_ unproductive non-terminals and errors for
        _definite_ unproductive non-terminals.  Volunteers?
        
        XXX: INCOMPLETE
        """
        def recurse(a):
            if a == 5:
                return a
            return recurse(a+1)

        grammarUnchecked = dict([(rule, "".join(rhs)) for (rule, rhs) in nameGrammar.items()])
        grammarProductive = []
        finished = False
        while not finished:
            print "grammarProductive:"
            print grammarProductive
            print "grammarUnchecked:"
            print grammarUnchecked
            print
            finished = True
            for rule, rhs in grammarUnchecked.items():
                matchNonTerminal = reNonTerminal.search(rhs)
                while matchNonTerminal:
                    matchString = matchNonTerminal.group(1)
                    if matchString not in grammarProductive:
                        break
                    rhs = rhs.replace("<"+matchString+">", "")
                    finished = False
                    matchNonTerminal = reNonTerminal.search(rhs)
                if not matchNonTerminal:
                    grammarProductive.append(rule)
                    del grammarUnchecked[rule]
                    finished = False
                    continue
                grammarUnchecked[rule] = rhs

    # More grammar checking functions to come:
    #   Unused non-terminals
    # Loop detection would be nice, but currently a little impractical.
    
    def checkUnusedNonTerminals(self, nameGrammar):
        """Check grammar for non-terminals that can never be reached.
        
        While unused non-terminals are irrelevant in the generation of sentences,
        their presence usually implies an error in the grammar.
        
        XXX: INCOMPLETE
        """
        
        pass

    # verifyGrammar() uses the above functions to verify the correctness of a
    # grammar.  This isn't perfect, but it should catch the most common problems.
    def verifyGrammar(self):
        error = self.checkUndefinedNonTerminals(self.grammar)
        if error:
            return stringUndefinedNonTerminal % error
        if "name" not in self.grammar:
            return "Rule \"name\" not present!"

    # Now to the meat of the problem, which is actually almost trivial thanks to
    # the dictionary data type.  I love python ;-)

    def getName(self):
        nameStr = random.choice(self.grammar["name"])
        matchNonTerminal = self.reNonTerminal.search(nameStr)
        while matchNonTerminal:
            subStr = random.choice(self.grammar[matchNonTerminal.group(1)])
            nameStr = self.reNonTerminal.sub(subStr, nameStr, 1)
            matchNonTerminal = self.reNonTerminal.search(nameStr)
        return nameStr


if __name__ == "__main__":
    # Main body
    # Test grammar -- will be read from a file when I decide how to do it properly
    # with minimum effort (for the user and the code)
    orkGrammar = {
        "name": ["<nameStart><nameMiddle0to3><nameEnd>"],
        "nameMiddle0to3": ["","<nameMiddle>", "<nameMiddle><nameMiddle>", "<nameMiddle><nameMiddle><nameMiddle>"],
        "nameStart": ["<nsCons><nmVowel>", "<nsCons><nmVowel>", "<nsCons><nmVowel>", "<nsVowel>"],
        "nameMiddle": ["<nmCons><nmVowel>"],
        "nameEnd": ["<neCons><neVowel>", "<neCons>", "<neCons>"],
        "nsCons": ["D", "G", "K", "T", "Gr"],
        "nmCons": ["d", "g", "k", "t", "r", "s", "z", "kt", "rs", "gr"],
        "neCons": ["r", "s", "z"],
        "nsVowel": ["E", "U"],
        "nmVowel": ["a", "e", "i", "o", "u"],
        "neVowel": ["a", "u"]
        }
    
    fooGrammar = {
        "name": ["<nameStart><nameMiddle0to2><nameEnd>"],
        "nameMiddle0to2": ["","<nameMiddle>", "<nameMiddle><nameMiddle>"],
        "nameStart": ["<nsCons><nmVowel>", "<nsCons><nmVowel>", "<nsCons><nmVowel>", "<nsVowel>"],
        "nameMiddle": ["<nmCons><nmVowel>"],
        "nameEnd": ["<neCons><neVowel>", "<neCons>", "<neCons>"],
        "nsCons": ["J", "M", "P", "N", "Y", "D", "F"],
        "nmCons": ["l", "m", "lm", "th", "r", "s", "ss", "p", "f", "mb", "b", "lb", "d", "lf"],
        "neCons": ["r", "n", "m", "s", "y", "l", "th", "b", "lb", "f", "lf"],
        "nsVowel": ["A", "Au", "Ei"],
        "nmVowel": ["a", "e", "i", "o", "u", "au", "oa", "ei"],
        "neVowel": ["e", "i", "a", "au"]
        }

    print "Ork Grammar:", yaml.dump(orkGrammar)
    print "Foo Grammar:", yaml.dump(fooGrammar)
    fooGen = CFNameGen(fooGrammar)
    errorStr = fooGen.verifyGrammar()
    if errorStr:
        sys.exit(errorStr)
    print fooGen.getName()

    orkGen = CFNameGen(orkGrammar)
    errorStr = orkGen.verifyGrammar()
    if errorStr:
        sys.exit(errorStr)
    print orkGen.getName()



#!/usr/bin/env python
# encoding: utf-8

#    Copyright © 2008 Arne Babenhauserheide
# 
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

"""rpg-1d6 - The 1d6 RPG backend module provides character management and interaction.  

Info: http://1w6.org """

from setuptools import setup, find_packages
from ews.__init__ import __doc__ as babtools__doc__
from ews.__init__ import __version__


def read_changelog():
    """Read and return the Changelog"""
    f = open("Changelog.txt", "r")
    log = f.read()
    f.close()
    return log

__changelog__ = read_changelog()

# Create the desription from the docstrings 

DESCRIPTION = __doc__.split("\n")[0].split(" - ")[1:]

LONG_DESCRIPTION = "\n".join(__doc__.split("\n")[1:])

LONG_DESCRIPTION += "\n\n" + babtools__doc__ 

LONG_DESCRIPTION += "\n\n" + __changelog__

setup(name=__doc__.split("\n")[0].split(" - ")[0],
      version=__version__,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION, 
      author='Arne Babenhauserheide',
      author_email='arne_bab@web.de',
      keywords=["ews", "rpg", "characters"], 
      license="GNU GPL-3 or later", 
      platforms=["any"], 
      requires = [], 
      classifiers = [
            "License :: OSI Approved :: GNU General Public License (GPL)",
            "Programming Language :: Python",
            "Operating System :: OS Independent",
            "Topic :: Software Development :: Libraries :: Python Modules",
            "Intended Audience :: Developers", 
            "Intended Audience :: End Users/Desktop", 
            "Environment :: Console", 
            "Development Status :: 3 - Alpha"
            ],
      url='http://1w6.org/programme',
      packages = find_packages('.'), 
      #py_modules=['babtools_gentoo'],
      #scripts=["babscript.py"]
     )
